@@ -1071,7 +1071,7 @@ ObtainTokens( PLUID lpLogonId,
             p = principal + tlen;
             *p++ = '@';
             StringCchCopy(p, len - tlen -1, pOpt->realm ? pOpt->realm : realm);
-            code = KFW_AFS_get_cred(principal, cell, password, 0, pOpt->smbName, preason);
+            code = KFW_AFS_get_cred(principal, cell, password, 0, NULL, preason);
             DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",
                         principal, pOpt->smbName, cell, code);
 
@@ -1080,7 +1080,7 @@ ObtainTokens( PLUID lpLogonId,
                 while ( *p ) {
                     if ( cm_stricmp_utf8(p, cell) ) {
                         SetEnvironmentVariable(DO_NOT_REGISTER_VARNAME, "");
-                        code2 = KFW_AFS_get_cred(principal, p, password, 0, pOpt->smbName, preason);
+                        code2 = KFW_AFS_get_cred(principal, p, password, 0, NULL, preason);
                         SetEnvironmentVariable(DO_NOT_REGISTER_VARNAME, NULL);
                         DebugEvent("KFW_AFS_get_cred  uname=[%s] smbname=[%s] cell=[%s] code=[%d]",
                                     principal, pOpt->smbName, p, code2);
@@ -1095,7 +1095,7 @@ ObtainTokens( PLUID lpLogonId,
 
     } else {
         code = ka_UserAuthenticateGeneral2(KA_USERAUTH_VERSION+KA_USERAUTH_AUTHENT_LOGON,
-                                            uname, "", cell, password, pOpt->smbName, 0, &pw_exp, 0,
+                                            uname, "", cell, password, NULL, 0, &pw_exp, 0,
                                             preason);
         DebugEvent("AFS AfsLogon - (INTEGRATED only)ka_UserAuthenticateGeneral2 Code[%x] uname[%s] smbname=[%s] Cell[%s] PwExp=[%d] Reason=[%s]",
                     code, uname, pOpt->smbName, cell, pw_exp, *preason ? *preason : "");
