@@ -66,13 +66,15 @@ static void CleanupACLEnt(cm_aclent_t * aclp)
 
 /*
  * Get an acl cache entry for a particular user and file, or return that it doesn't exist.
- * Called with the scp locked.
+ * Called with the scp write locked.
  */
 long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 *rightsp)
 {
     cm_aclent_t *aclp;
     long retval = -1;
+    time_t now = time(NULL);
 
+    lock_AssertWrite(&scp->rw);
     lock_ObtainWrite(&cm_aclLock);
     *rightsp = 0;   /* get a new acl from server if we don't find a
                      * current entry
@@ -80,7 +82,7 @@ long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 *rightsp)
 
     for (aclp = scp->randomACLp; aclp; aclp = aclp->nextp) {
         if (aclp->userp == userp) {
-            if (aclp->tgtLifetime && aclp->tgtLifetime <= time(NULL)) {
+            if (aclp->tgtLifetime && aclp->tgtLifetime <= now) {
                 /* ticket expired */
                 osi_QRemoveHT((osi_queue_t **) &cm_data.aclLRUp, (osi_queue_t **) &cm_data.aclLRUEndp, &aclp->q);
                 CleanupACLEnt(aclp);
@@ -91,13 +93,13 @@ long cm_FindACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 *rightsp)
                            &aclp->q);
             } else {
                 *rightsp = aclp->randomAccess;
-		if (cm_data.aclLRUp != aclp) {
-		    /* move to the head of the LRU queue */
-		    osi_QRemoveHT((osi_queue_t **) &cm_data.aclLRUp, (osi_queue_t **) &cm_data.aclLRUEndp, &aclp->q);
-		    osi_QAddH((osi_queue_t **) &cm_data.aclLRUp,
-			       (osi_queue_t **) &cm_data.aclLRUEndp,
-			       &aclp->q);
-		}
+                if (cm_data.aclLRUp != aclp) {
+                    /* move to the head of the LRU queue */
+                    osi_QRemoveHT((osi_queue_t **) &cm_data.aclLRUp, (osi_queue_t **) &cm_data.aclLRUEndp, &aclp->q);
+                    osi_QAddH((osi_queue_t **) &cm_data.aclLRUp,
+                              (osi_queue_t **) &cm_data.aclLRUEndp,
+                              &aclp->q);
+                }
                 retval = 0;     /* success */
             }
             break;
@@ -139,6 +141,22 @@ static cm_aclent_t *GetFreeACLEnt(cm_scache_t * scp)
     return aclp;
 }
 
+time_t cm_TGTLifeTime(cm_user_t *userp, afs_uint32 cellID)
+{
+    cm_cell_t *cellp = NULL;
+    cm_ucell_t * ucp = NULL;
+    time_t      expirationTime = 0;
+
+    lock_ObtainMutex(&userp->mx);
+    cellp = cm_FindCellByID(cellID, CM_FLAG_NOPROBE);
+    ucp = cm_GetUCell(userp, cellp);
+    if (ucp->ticketp)
+        expirationTime = ucp->expirationTime;
+    lock_ReleaseMutex(&userp->mx);
+
+    return expirationTime;
+}
+
 
 /*
  * Add rights to an acl cache entry.  Do the right thing if not present,
@@ -149,13 +167,23 @@ static cm_aclent_t *GetFreeACLEnt(cm_scache_t * scp)
 long cm_AddACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 rights)
 {
     struct cm_aclent *aclp;
+    time_t tgtLifeTime;
+
+    tgtLifeTime = cm_TGTLifeTime(userp, scp->fid.cell);
 
     lock_ObtainWrite(&cm_aclLock);
     for (aclp = scp->randomACLp; aclp; aclp = aclp->nextp) {
         if (aclp->userp == userp) {
             aclp->randomAccess = rights;
-            if (aclp->tgtLifetime == 0)
-                aclp->tgtLifetime = cm_TGTLifeTime(pag);
+            if (aclp->tgtLifetime < tgtLifeTime)
+                aclp->tgtLifetime = tgtLifeTime;
+            if (cm_data.aclLRUp != aclp) {
+                /* move to the head of the LRU queue */
+                osi_QRemoveHT((osi_queue_t **) &cm_data.aclLRUp, (osi_queue_t **) &cm_data.aclLRUEndp, &aclp->q);
+                osi_QAddH((osi_queue_t **) &cm_data.aclLRUp,
+                           (osi_queue_t **) &cm_data.aclLRUEndp,
+                           &aclp->q);
+            }
             lock_ReleaseWrite(&cm_aclLock);
             return 0;
         }
@@ -174,7 +202,7 @@ long cm_AddACLCache(cm_scache_t *scp, cm_user_t *userp, afs_uint32 rights)
     cm_HoldUser(userp);
     aclp->userp = userp;
     aclp->randomAccess = rights;
-    aclp->tgtLifetime = cm_TGTLifeTime(userp);
+    aclp->tgtLifetime = tgtLifeTime;
     lock_ReleaseWrite(&cm_aclLock);
 
     return 0;
@@ -327,6 +355,7 @@ void cm_InvalidateACLUser(cm_scache_t *scp, cm_user_t *userp)
     cm_aclent_t *aclp;
     cm_aclent_t **laclpp;
     int found = 0;
+    int callback = 0;
 
     lock_ObtainWrite(&scp->rw);
     lock_ObtainWrite(&cm_aclLock);
@@ -338,14 +367,17 @@ void cm_InvalidateACLUser(cm_scache_t *scp, cm_user_t *userp)
             aclp->userp = NULL;
             aclp->backp = (struct cm_scache *) 0;
             found = 1;
-            if (RDR_Initialized && cm_HaveCallback(scp))
-                RDR_InvalidateObject(scp->fid.cell, scp->fid.volume, scp->fid.vnode, scp->fid.unique,
-                                     scp->fid.hash, scp->fileType, AFS_INVALIDATE_CREDS);
             break;
         }
     }
     lock_ReleaseWrite(&cm_aclLock);
+    if (found)
+        callback = cm_HaveCallback(scp);
     lock_ReleaseWrite(&scp->rw);
+
+    if (found && callback && RDR_Initialized)
+        RDR_InvalidateObject(scp->fid.cell, scp->fid.volume, scp->fid.vnode, scp->fid.unique,
+                             scp->fid.hash, scp->fileType, AFS_INVALIDATE_CREDS);
 }
 
 /*
@@ -374,6 +406,8 @@ cm_ResetACLCache(cm_cell_t *cellp, cm_user_t *userp)
     }
     lock_ReleaseRead(&cm_scacheLock);
 
+    cm_EAccesClearUserEntries(userp, cellp ? cellp->cellID : 0);
+
     if (RDR_Initialized) {
         lock_ObtainRead(&cm_volumeLock);
         for (hash = 0; hash < cm_data.volumeHashTableSize; hash++) {
