@@ -31,6 +31,7 @@ static cm_localMountPoint_t* cm_localMountPoints;
 osi_mutex_t cm_Freelance_Lock;
 static int cm_localMountPointChangeFlag = 0;
 int cm_freelanceEnabled = 1;
+int cm_freelanceDiscovery = 1;
 int cm_freelanceImportCellServDB = 0;
 time_t FakeFreelanceModTime = 0x3b49f6e2;
 
@@ -398,6 +399,9 @@ int cm_noteLocalMountPointChange(afs_int32 locked) {
     cm_data.fakeDirVersion++;
     cm_localMountPointChangeFlag = 1;
 
+    if (!locked)
+        lock_ReleaseMutex(&cm_Freelance_Lock);
+
     if (RDR_Initialized) {
         cm_fid_t fid;
         cm_FakeRootFid(&fid);
@@ -405,9 +409,6 @@ int cm_noteLocalMountPointChange(afs_int32 locked) {
                               CM_SCACHETYPE_DIRECTORY,
                               AFS_INVALIDATE_DATA_VERSION);
     }
-
-    if (!locked)
-        lock_ReleaseMutex(&cm_Freelance_Lock);
     return 1;
 }
 
@@ -1535,13 +1536,14 @@ long
 cm_FreelanceFetchMountPointString(cm_scache_t *scp)
 {
     lock_ObtainMutex(&cm_Freelance_Lock);
-    if (!scp->mountPointStringp[0] &&
+    if (scp->mpDataVersion != scp->dataVersion &&
         scp->fid.cell == AFS_FAKE_ROOT_CELL_ID &&
         scp->fid.volume == AFS_FAKE_ROOT_VOL_ID &&
         (afs_int32)(scp->fid.unique - cm_data.fakeUnique) - 1 >= 0 &&
         scp->fid.unique - cm_data.fakeUnique <= cm_noLocalMountPoints) {
         strncpy(scp->mountPointStringp, cm_localMountPoints[scp->fid.unique-cm_data.fakeUnique-1].mountPointStringp, MOUNTPOINTLEN);
         scp->mountPointStringp[MOUNTPOINTLEN-1] = 0;	/* null terminate */
+        scp->mpDataVersion = scp->dataVersion;
     }
     lock_ReleaseMutex(&cm_Freelance_Lock);
 
