@@ -163,13 +163,16 @@ typedef struct _AFS_NONPAGED_FCB
     USHORT          Type;
 
     //
-    // Ranking - File Resource first, then Paging Resource
+    // Ranking - File Resource, Paging Resource,
+    //           then SectionObject Resource
     //
 
     ERESOURCE       Resource;
 
     ERESOURCE       PagingResource;
 
+    ERESOURCE       SectionObjectResource;
+
     //
     // The section object pointer
     //
@@ -376,12 +379,6 @@ typedef struct AFS_FCB
 
             LONG                ExtentCount;
 
-            //
-            // The Lazy writer thread
-            //
-
-            PETHREAD            LazyWriterThread;
-
             //
             // Current count of queued flush items for the file
             //
@@ -775,7 +772,7 @@ void *
 
 typedef
 void
-(*PAFSExFreePool)( IN void *Pointer);
+(*PAFSExFreePoolWithTag)( IN void *Pointer, IN ULONG Tag);
 
 typedef
 void
@@ -792,6 +789,8 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
     UNICODE_STRING      AFSServerName;
 
+    UNICODE_STRING      AFSMountRootName;
+
     ULONG               AFSDebugFlags;
 
     AFSFileID           GlobalRootFid;
@@ -814,7 +813,7 @@ typedef struct _AFS_LIBRARY_INIT_CB
 
     PAFSExAllocatePoolWithTag   AFSExAllocatePoolWithTag;
 
-    PAFSExFreePool      AFSExFreePool;
+    PAFSExFreePoolWithTag      AFSExFreePoolWithTag;
 
     PAFSDumpTraceFiles  AFSDumpTraceFiles;
 
