@@ -109,7 +109,7 @@
 
 
 int cache_bypass_strategy   = 	NEVER_BYPASS_CACHE;
-afs_size_t cache_bypass_threshold  =  	AFS_CACHE_BYPASS_DISABLED; /* file size > threshold triggers bypass */
+int cache_bypass_threshold  =  	AFS_CACHE_BYPASS_DISABLED; /* file size > threshold triggers bypass */
 int cache_bypass_prefetch = 1;	/* Should we do prefetching ? */
 
 extern afs_rwlock_t afs_xcbhash;
@@ -270,12 +270,49 @@ done:
 #ifdef UKERNEL
 typedef void * bypass_page_t;
 
+#define copy_page(pp, pageoff, rxiov, iovno, iovoff, auio, curiov)	\
+    do { \
+	int dolen = auio->uio_iov[curiov].iov_len - pageoff; \
+	memcpy(((char *)pp) + pageoff,		       \
+	       ((char *)rxiov[iovno].iov_base) + iovoff, dolen);	\
+	auio->uio_resid -= dolen; \
+    } while(0)
+
+#define copy_pages(pp, pageoff, rxiov, iovno, iovoff, auio, curiov)	\
+    do { \
+	int dolen = rxiov[iovno].iov_len - iovoff; \
+	memcpy(((char *)pp) + pageoff,				\
+	       ((char *)rxiov[iovno].iov_base) + iovoff, dolen);	\
+	auio->uio_resid -= dolen;	\
+    } while(0)
+
 #define unlock_and_release_pages(auio)
 #define release_full_page(pp, pageoff)
 
 #else
 typedef struct page * bypass_page_t;
 
+#define copy_page(pp, pageoff, rxiov, iovno, iovoff, auio, curiov)	\
+    do { \
+        char *address;						\
+	int dolen = auio->uio_iov[curiov].iov_len - pageoff; \
+	address = kmap_atomic(pp, KM_USER0); \
+	memcpy(address + pageoff, \
+	       (char *)(rxiov[iovno].iov_base) + iovoff, dolen);	\
+	kunmap_atomic(address, KM_USER0); \
+    } while(0)
+
+#define copy_pages(pp, pageoff, rxiov, iovno, iovoff, auio, curiov)	\
+    do { \
+        char *address; \
+	int dolen = rxiov[iovno].iov_len - iovoff; \
+	address = kmap_atomic(pp, KM_USER0); \
+	memcpy(address + pageoff, \
+	       (char *)(rxiov[iovno].iov_base) + iovoff, dolen);	\
+	kunmap_atomic(address, KM_USER0); \
+    } while(0)
+
+
 #define unlock_and_release_pages(auio) \
     do { \
 	struct iovec *ciov;	\
@@ -310,38 +347,8 @@ typedef struct page * bypass_page_t;
 	    afs_warn("afs_NoCacheFetchProc: page not locked!\n"); \
 	put_page(pp); /* decrement refcount */ \
     } while(0)
-#endif
 
-static void
-afs_bypass_copy_page(bypass_page_t pp, int pageoff, struct iovec *rxiov,
-	int iovno, int iovoff, struct uio *auio, int curiov, int partial)
-{
-    char *address;
-    int dolen;
-
-    if (partial)
-	dolen = rxiov[iovno].iov_len - iovoff;
-    else
-	dolen = auio->uio_iov[curiov].iov_len - pageoff;
-
-#if !defined(UKERNEL)
-# if defined(KMAP_ATOMIC_TAKES_NO_KM_TYPE)
-    address = kmap_atomic(pp);
-# else
-    address = kmap_atomic(pp, KM_USER0);
-# endif
-#else
-    address = pp;
 #endif
-    memcpy(address + pageoff, (char *)(rxiov[iovno].iov_base) + iovoff, dolen);
-#if !defined(UKERNEL)
-# if defined(KMAP_ATOMIC_TAKES_NO_KM_TYPE)
-    kunmap_atomic(address);
-# else
-    kunmap_atomic(address, KM_USER0);
-# endif
-#endif
-}
 
 /* no-cache prefetch routine */
 static afs_int32
@@ -440,7 +447,7 @@ afs_NoCacheFetchProc(struct rx_call *acall,
 		if (pageoff + (rxiov[iovno].iov_len - iovoff) <= auio->uio_iov[curpage].iov_len) {
 		    /* Copy entire (or rest of) current iovec into current page */
 		    if (pp)
-			afs_bypass_copy_page(pp, pageoff, rxiov, iovno, iovoff, auio, curpage, 0);
+			copy_pages(pp, pageoff, rxiov, iovno, iovoff, auio, curpage);
 		    length -= (rxiov[iovno].iov_len - iovoff);
 		    pageoff += rxiov[iovno].iov_len - iovoff;
 		    iovno++;
@@ -448,7 +455,7 @@ afs_NoCacheFetchProc(struct rx_call *acall,
 		} else {
 		    /* Copy only what's needed to fill current page */
 		    if (pp)
-			afs_bypass_copy_page(pp, pageoff, rxiov, iovno, iovoff, auio, curpage, 1);
+			copy_page(pp, pageoff, rxiov, iovno, iovoff, auio, curpage);
 		    length -= (auio->uio_iov[curpage].iov_len - pageoff);
 		    iovoff += auio->uio_iov[curpage].iov_len - pageoff;
 		    pageoff = auio->uio_iov[curpage].iov_len;
@@ -485,7 +492,7 @@ afs_ReadNoCache(struct vcache *avc,
     /* the reciever will free this */
     areq = osi_Alloc(sizeof(struct vrequest));
 
-    if (avc->vc_error) {
+    if (avc && avc->vc_error) {
 	code = EIO;
 	afs_warn("afs_ReadNoCache VCache Error!\n");
 	goto cleanup;
@@ -575,7 +582,7 @@ afs_PrefetchNoCache(struct vcache *avc,
     iovecp = auio->uio_iov;
 #endif
 
-    tcallspec = osi_Alloc(sizeof(struct tlocal1));
+    tcallspec = (struct tlocal1 *) osi_Alloc(sizeof(struct tlocal1));
     do {
 	tc = afs_Conn(&avc->f.fid, areq, SHARED_LOCK /* ignored */, &rxconn);
 	if (tc) {
