@@ -33,6 +33,7 @@
 #define USING_HEIMDAL 1
 #endif
 
+#include "assert.h"
 #include "skipwrap.h"
 
 /* This code borrowed heavily from the previous version of log.  Here is the
@@ -682,7 +683,7 @@ CommandProc(struct cmd_syndesc *as, void *arock)
 
         if (get_cred_keylen(afscred) != sizeof(atoken->sessionKey)) {
             afs_com_err(rn, 0, "Invalid rxkad key length (%u != 8) key type (%u)",
-                        (unsigned)get_cred_keylen(afscred),
+                        get_cred_keylen(afscred),
                         get_creds_enctype(afscred));
             KLOGEXIT(1);
         }
