@@ -10,10 +10,12 @@
 #ifndef _BOSPROTOTYPES_H_
 #define _BOSPROTOTYPES_H_
 
+#include <rx/rxkad.h>
+
 /* bnode.c */
 int bnode_CoreName(struct bnode *abnode, char *acoreName, char *abuffer);
 int bnode_GetString(struct bnode *abnode, char *abuffer, afs_int32 alen);
-int bnode_GetParm(struct bnode *abnode, afs_int32 aindex, char *abuffer, 
+int bnode_GetParm(struct bnode *abnode, afs_int32 aindex, char *abuffer,
 		  afs_int32 alen);
 int bnode_GetStat(struct bnode *abnode, afs_int32 * astatus);
 int bnode_RestartP(struct bnode *abnode);
@@ -41,9 +43,11 @@ int bnode_Deactivate(struct bnode *abnode);
 void bozo_Log(char *format, ... );
 int bozo_ReBozo(void);
 int WriteBozoFile(char *aname);
+int bozo_CreatePidFile(char *ainst, char *aname, pid_t apid);
+int bozo_DeletePidFile(char *ainst, char *aname);
 
 /* bosoprocs.c */
-int GetRequiredDirPerm(const char *path);	
+int GetRequiredDirPerm(const char *path);
 void *bozo_ShutdownAndExit(void *arock /* really int asignal */);
 int initBosEntryStats(void);
 int DirAccessOK(void);
@@ -51,7 +55,10 @@ int DirAccessOK(void);
 /* fsbnodeops.c */
 char *copystr(char *a);
 
-/* bosoprocs.c */
-void *bozo_ShutdownAndExit(void *arock /* really int asignal */);
+/* inline functions */
+static_inline struct bozo_key *
+ktc_to_bozoptr(struct ktc_encryptionKey *key) {
+    return (struct bozo_key *)key;
+}
 
 #endif
