@@ -110,7 +110,7 @@ GetText(struct rx_call *call, afs_uint32 lockHandle, afs_int32 textType,
 
     /* compute minimum of remaining text or user buffer */
     textRemaining = ntohl(tbPtr->size) - offset;
-    transferSize = min(textRemaining, maxLength);
+    transferSize = MIN(textRemaining, maxLength);
 
     /* allocate the transfer storage */
     if (transferSize <= 0) {
@@ -118,7 +118,7 @@ GetText(struct rx_call *call, afs_uint32 lockHandle, afs_int32 textType,
 	charListPtr->charListT_val = NULL;
     } else {
 	charListPtr->charListT_len = transferSize;
-	charListPtr->charListT_val = malloc(transferSize);
+	charListPtr->charListT_val = (char *)malloc(transferSize);
 	if (charListPtr->charListT_val == 0)
 	    ABORT(BUDB_NOMEM);
     }
@@ -177,7 +177,7 @@ GetText(struct rx_call *call, afs_uint32 lockHandle, afs_int32 textType,
 
   no_xfer_abort:
     charListPtr->charListT_len = 0;
-    charListPtr->charListT_val = NULL;
+    charListPtr->charListT_val = (char *)malloc(0);
 
   abort_exit:
     if (ut)
@@ -365,7 +365,7 @@ SaveText(struct rx_call *call, afs_uint32 lockHandle, afs_int32 textType,
     while (textLength) {
 	/* compute the transfer size */
 	remainingInBlock = (BLOCK_DATA_SIZE - (offset % BLOCK_DATA_SIZE));
-	chunkSize = min(remainingInBlock, textLength);
+	chunkSize = MIN(remainingInBlock, textLength);
 
 	/* copy in the data */
 	memcpy(&diskBlock.a[offset % BLOCK_DATA_SIZE], textptr, chunkSize);
@@ -466,7 +466,7 @@ saveTextToFile(struct ubik_trans *ut, struct textBlock *tbPtr)
     size = ntohl(tbPtr->size);
     blockAddr = ntohl(tbPtr->textAddr);
     while (size) {
-	chunkSize = min(BLOCK_DATA_SIZE, size);
+	chunkSize = MIN(BLOCK_DATA_SIZE, size);
 	dbread(ut, blockAddr, (char *)&block, sizeof(block));
 	write(fid, &block.a[0], chunkSize);
 	blockAddr = ntohl(block.h.next);
