@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -33,8 +33,8 @@ struct buServerConfS {
     FILE *log;			/* log file for status/errors */
 
     /* ubik and comm. related */
-    afs_int32 myHost;
-    afs_int32 serverList[MAXSERVERS];	/* list of ubik servers */
+    afs_uint32 myHost;
+    afs_uint32 serverList[MAXSERVERS];	/* list of ubik servers */
     char *cellConfigdir;	/* afs cell config. directory */
     struct ubik_dbase *database;	/* handle for the database */
 
