@@ -146,7 +146,7 @@ STC_LabelTape(struct rx_call *acid, struct tc_tapeLabel *label, afs_uint32 *task
     if (callPermitted(acid) == 0)
 	return (TC_NOTPERMITTED);
 
-    ptr = malloc(sizeof(*ptr));
+    ptr = (struct labelTapeIf *)malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     memcpy(&ptr->label, label, sizeof(ptr->label));
@@ -226,13 +226,18 @@ STC_PerformDump(struct rx_call *rxCallId, struct tc_dumpInterface *tcdiPtr, tc_d
     /*set up the parameters in the node, to be used by LWP */
     strcpy(newNode->dumpSetName, tcdiPtr->dumpName);
 
-    newNode->dumpName = strdup(tcdiPtr->dumpPath);
-    newNode->volumeSetName = strdup(tcdiPtr->volumeSetName);
+    newNode->dumpName = (char *)malloc(strlen(tcdiPtr->dumpPath) + 1);
+    strcpy(newNode->dumpName, tcdiPtr->dumpPath);
+
+    newNode->volumeSetName =
+	(char *)malloc(strlen(tcdiPtr->volumeSetName) + 1);
+    strcpy(newNode->volumeSetName, tcdiPtr->volumeSetName);
 
     CopyTapeSetDesc(&(newNode->tapeSetDesc), &tcdiPtr->tapeSet);
 
-    newNode->dumps = malloc(sizeof(struct tc_dumpDesc) *
-			    tc_dumpArrayPtr->tc_dumpArray_len);
+    newNode->dumps = (struct tc_dumpDesc *)
+	malloc(sizeof(struct tc_dumpDesc) *
+	       tc_dumpArrayPtr->tc_dumpArray_len);
     newNode->arraySize = tc_dumpArrayPtr->tc_dumpArray_len;
     CopyDumpDesc(newNode->dumps, tc_dumpArrayPtr);
 
@@ -313,8 +318,9 @@ STC_PerformRestore(struct rx_call *acid, char *dumpSetName, tc_restoreArray *are
     /* this creates a node in list, alots an id for it and prepares it for locking */
     CreateNode(&newNode);
 
-    newNode->restores = malloc(sizeof(struct tc_restoreDesc) *
-		               arestores->tc_restoreArray_len);
+    newNode->restores = (struct tc_restoreDesc *)
+	malloc(sizeof(struct tc_restoreDesc) *
+	       arestores->tc_restoreArray_len);
     newNode->arraySize = arestores->tc_restoreArray_len;
     CopyRestoreDesc(newNode->restores, arestores);
     *taskID = newNode->taskID;
@@ -475,7 +481,7 @@ STC_SaveDb(struct rx_call *rxCall, Date archiveTime, afs_uint32 *taskId)
 
     *taskId = allocTaskId();
 
-    ptr = malloc(sizeof(struct saveDbIf));
+    ptr = (struct saveDbIf *)malloc(sizeof(struct saveDbIf));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     ptr->archiveTime = archiveTime;
@@ -554,7 +560,7 @@ STC_ScanDumps(struct rx_call *acid, afs_int32 addDbFlag, afs_uint32 *taskId)
 
     *taskId = allocTaskId();
 
-    ptr = malloc(sizeof(*ptr));
+    ptr = (struct scanTapeIf *)malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
     ptr->addDbFlag = addDbFlag;
@@ -642,7 +648,7 @@ STC_DeleteDump(struct rx_call *acid, afs_uint32 dumpID, afs_uint32 *taskId)
     if (callPermitted(acid) == 0)
 	return (TC_NOTPERMITTED);
 
-    ptr = malloc(sizeof(*ptr));
+    ptr = (struct deleteDumpIf *)malloc(sizeof(*ptr));
     if (!ptr)
 	ERROR_EXIT(TC_NOMEMORY);
 
