@@ -4,7 +4,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -24,7 +24,7 @@
 #define	__alpha		1
 #define	AFS_ALPHA_ENV	1
 #define	AFS_DECOSF_ENV	1
-#define	AFS_64BIT_ENV	1
+#define AFS_BOZONLOCK_ENV       1
 #define AFS_64BIT_CLIENT	1
 
 #include <afs/afs_sysnames.h>
@@ -129,7 +129,7 @@ enum vcexcl { NONEXCL, EXCL };
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -146,7 +146,6 @@ enum vcexcl { NONEXCL, EXCL };
 
 #define UKERNEL			1	/* user space kernel */
 #define AFS_GREEDY43_ENV	1	/* Used only in rx/rx_user.c */
-#define AFS_64BIT_ENV		1
 #define AFS_ENV			1
 #define AFS_USR_OSF_ENV		1
 #define AFS_USR_DUX40_ENV	1
@@ -166,14 +165,13 @@ enum vcexcl { NONEXCL, EXCL };
 #define SYS_NAME	"alpha_dux51"
 #define SYS_NAME_ID	SYS_NAME_ID_alpha_dux51
 #define AFS_HAVE_FFS            1	/* Use system's ffs. */
-#define AFS_HAVE_STATVFS	0	/* System doesn't support statvfs */
+#define AFS_HAVE_STATVFS	1	/* System supports statvfs */
 #define AFSLITTLE_ENDIAN        1
 
 /* Extra kernel definitions (from kdefs file) */
 #ifdef KERNEL
 #define	AFS_UIOFMODE		1	/* Only in afs/afs_vnodeops.c (afs_ustrategy) */
 #define	AFS_SYSVLOCK		1	/* sys v locking supported */
-/*#define	AFS_USEBUFFERS	1*/
 #define	afsio_iov	uio_iov
 #define	afsio_iovcnt	uio_iovcnt
 #define	afsio_offset	uio_offset
