@@ -28,7 +28,6 @@
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
-#include <rx/rxkad_convert.h>
 #include <lock.h>
 #define UBIK_LEGACY_CALLITER 1
 #include <ubik.h>
@@ -1522,9 +1521,10 @@ MyBeforeProc(struct cmd_syndesc *as, void *arock)
 			afs_com_err(whoami, code, "prompting for %s", p + 1);
 			return code;
 		    }
-		    ip = malloc(sizeof(struct cmd_item));
-		    ip->data = strdup(password);
+		    ip = (struct cmd_item *)malloc(sizeof(struct cmd_item));
+		    ip->data = (char *)malloc(strlen(password) + 1);
 		    ip->next = 0;
+		    strcpy(ip->data, password);
 		    as->parms[i].items = ip;
 		}
 	    }
