@@ -27,7 +27,6 @@
 
 #include <afs/pthread_glock.h>
 #include <rx/rxkad.h>
-#include <rx/rxkad_convert.h>
 #include <afs/cellconfig.h>
 #include <ubik.h>
 #include <afs/auth.h>
@@ -104,8 +103,8 @@ ka_GetServers(char *cell, struct afsconf_cell * cellinfo)
     char cellname[MAXKTCREALMLEN];
 
     LOCK_GLOBAL_MUTEX;
-    if (cell == NULL || strlen(cell) == 0)
-	cell = NULL;
+    if (cell && !strlen(cell))
+	cell = 0;
     else
 	cell = lcstring(cellname, cell, sizeof(cellname));
 
