@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -16,21 +16,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/libacl/aclprocs.c,v 1.14 2006/02/23 15:52:23 rees Exp $");
+#include <roken.h>
 
-#include <sys/types.h>
-#ifdef AFS_NT40_ENV
-#include <winsock2.h>
-#else
-#include <netinet/in.h>
-#endif
-#ifdef HAVE_STRING_H
-#include <string.h>
-#endif
 #include <rx/xdr.h>
 #include <rx/rx.h>
 #include <afs/ptclient.h>
+#include <afs/ptuser.h>
+
 #include "acl.h"
 
 #ifdef AFS_PTHREAD_ENV
@@ -39,9 +31,6 @@ RCSID
 pthread_mutex_t acl_list_mutex;
 #endif /* AFS_PTHREAD_ENV */
 
-static int AddToList();
-static int GetFromList();
-
 struct freeListEntry {
     struct freeListEntry *next;
     int size;
@@ -50,11 +39,16 @@ struct freeListEntry {
 
 struct freeListEntry *freeList;
 
+static int AddToList(struct freeListEntry **, struct freeListEntry *);
+static int GetFromList(struct freeListEntry **, struct freeListEntry **,
+		       afs_int32);
+
 /*todo: for sorting acls - make sure they work with new groups lists 10/5*/
 static int
-CmpPlus(a, b)
-     struct acl_accessEntry *a, *b;
+CmpPlus(const void *arg1, const void *arg2)
 {
+    const struct acl_accessEntry *a = (struct acl_accessEntry *) arg1;
+    const struct acl_accessEntry *b = (struct acl_accessEntry *) arg2;
     if (a->id < b->id)
 	return (-1);
     if (a->id == b->id)
@@ -63,9 +57,11 @@ CmpPlus(a, b)
 }
 
 static int
-CmpMinus(a, b)
-     struct acl_accessEntry *a, *b;
+CmpMinus(const void *arg1, const void *arg2)
 {
+    const struct acl_accessEntry *a = (struct acl_accessEntry *) arg1;
+    const struct acl_accessEntry *b = (struct acl_accessEntry *) arg2;
+
     if (a->id > b->id)
 	return (-1);
     if (a->id == b->id)
@@ -74,8 +70,7 @@ CmpMinus(a, b)
 }
 
 static int
-CmpInt(x, y)
-     afs_int32 x, y;
+CmpInt(int x, int y)
 {
     if (x < y)
 	return (-1);
@@ -137,7 +132,7 @@ acl_NewExternalACL(int nEntries, char **r)
     struct freeListEntry *e;
 
     t = 20 + (nEntries) * (PR_MAXNAMELEN + 20);
-    /* Conservative estimate: enough space in each entry for longest 
+    /* Conservative estimate: enough space in each entry for longest
      * name plus decimal 2**32 (for largest rights mask) plus some formatting */
 
     if (GetFromList(&freeList, &e, t)) {
@@ -168,16 +163,19 @@ acl_FreeExternalACL(char **r)
     return (AddToList(&freeList, x));
 }
 
-
 int
-acl_Externalize(struct acl_accessList *acl, char **elist)
+acl_Externalize_pr(int (*func)(idlist *ids, namelist *names), struct acl_accessList *acl, char **elist)
 {
-    /* Converts the access list defined by acl into the external access list in elist.  Non-translatable id's are converted to their ASCII string representations.  Returns 0 on success, -1 if number of entries exceeds ACL_MAXENTRIES, or a failure code from the protection server if the problem occured there. */
+    /* Converts the access list defined by acl into the external access list
+     * in elist.  Non-translatable id's are converted to their ASCII string
+     * representations.  Returns 0 on success, -1 if number of entries
+     * exceeds ACL_MAXENTRIES, or a failure code from the protection server
+     * if the problem occured there. */
 
-    register int i;
-    register int j;
-    register code;
-    register char *nextc;
+    int i;
+    int j;
+    int code;
+    char *nextc;
     idlist lids;
     namelist lnames;
 
@@ -198,7 +196,7 @@ acl_Externalize(struct acl_accessList *acl, char **elist)
     j = i;
     for (i = acl->total - 1; i >= acl->total - acl->negative; i--, j++)
 	lids.idlist_val[j] = acl->entries[i].id;
-    code = pr_IdToName(&lids, &lnames);
+    code = (*func)(&lids, &lnames);
     if (code != 0) {
 	if (lids.idlist_val)
 	    free(lids.idlist_val);
@@ -226,15 +224,22 @@ acl_Externalize(struct acl_accessList *acl, char **elist)
     return (0);
 }
 
+int
+acl_Externalize(struct acl_accessList *acl, char **elist)
+{
+    return acl_Externalize_pr(pr_IdToName, acl, elist);
+}
 
 int
-acl_Internalize(char *elist, struct acl_accessList **acl)
+acl_Internalize_pr(int (*func)(namelist *names, idlist *ids), char *elist, struct acl_accessList **acl)
 {
-    /* Converts the external access list elist into the access list acl.  Returns 0 on success, -1 if ANY name is not translatable, or if the number of entries exceeds al_maxExtEntries. */
-    register int i;
-    register int j;
-    register char *nextc;
-    register afs_int32 code;
+    /* Converts the external access list elist into the access list acl.
+     * Returns 0 on success, -1 if ANY name is not translatable, or if
+     * the number of entries exceeds al_maxExtEntries. */
+    int i;
+    int j;
+    char *nextc;
+    afs_int32 code;
     int p, n;
     namelist lnames;
     idlist lids;
@@ -289,7 +294,7 @@ acl_Internalize(char *elist, struct acl_accessList **acl)
     lids.idlist_len = 0;
     lids.idlist_val = 0;
 
-    code = pr_NameToId(&lnames, &lids);
+    code = (*func)(&lnames, &lids);
     if (code) {
 	free(lnames.namelist_val);
 	if (lids.idlist_val)
@@ -326,6 +331,11 @@ acl_Internalize(char *elist, struct acl_accessList **acl)
     return (0);
 }
 
+int
+acl_Internalize(char *elist, struct acl_accessList **acl)
+{
+    return acl_Internalize_pr(pr_NameToId, elist, acl);
+}
 
 int
 acl_CheckRights(struct acl_accessList *acl, prlist *groups, int *rights)
@@ -426,9 +436,7 @@ acl_IsAMember(afs_int32 aid, prlist *cps)
 
 
 static int
-AddToList(pflist, elem)
-     struct freeListEntry **pflist;
-     struct freeListEntry *elem;
+AddToList(struct freeListEntry **pflist, struct freeListEntry *elem)
 {
     /* Adds elem to the freelist flist;  returns 0 */
 #ifdef AFS_PTHREAD_ENV
@@ -443,12 +451,13 @@ AddToList(pflist, elem)
 }
 
 static int
-GetFromList(pflist, elem, minsize)
-     struct freeListEntry **pflist;
-     struct freeListEntry **elem;
-     afs_int32 minsize;
+GetFromList(struct freeListEntry **pflist, struct freeListEntry **elem,
+	    afs_int32 minsize)
 {
-    /* Looks for an element whose body is at least minsize bytes in the freelist flist.  If found, unlinks it, puts its address in elem, and returns 0, else returns -1.  A trivial first-fit algorithm is used. */
+    /* Looks for an element whose body is at least minsize bytes in the
+     * freelist flist.  If found, unlinks it, puts its address in elem,
+     * and returns 0, else returns -1.  A trivial first-fit algorithm is
+     * used. */
 
     struct freeListEntry *y, *z;
 
