@@ -158,7 +158,7 @@ fd_set *IOMGR_AllocFDSet(void)
 	iomgrFreeFDSets = iomgrFreeFDSets->next;
     }
     else {
-	t = malloc(sizeof(fd_set));
+	t = (struct IOMGR_fd_set *)malloc(sizeof(fd_set));
     }
     if (!t)
 	return (fd_set*)0;
@@ -174,11 +174,11 @@ static struct IoRequest *NewRequest(void)
 {
     struct IoRequest *request;
 
-    if ((request=iorFreeList)) {
+    if ((request=iorFreeList))
 	iorFreeList = (struct IoRequest *) (request->next);
-	memset(request, 0, sizeof(struct IoRequest));
-    } else request = calloc(1, sizeof(struct IoRequest));
+    else request = (struct IoRequest *) malloc(sizeof(struct IoRequest));
 
+    memset((char*)request, 0, sizeof(struct IoRequest));
     return request;
 }
 
