@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -16,12 +16,13 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
+#include <roken.h>
 
 
 #include "lwp.h"
 #include "preempt.h"
 
-#if defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV) 
+#if defined(AFS_LINUX20_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_XBSD_ENV)
 int PRE_Block = 0;
 
 
@@ -38,8 +39,6 @@ PRE_EndPreempt(void)
 }
 
 #else
-#include <sys/time.h>
-#include <signal.h>
 #ifdef HAVE_UCONTEXT_H
 #include <ucontext.h>
 #endif
@@ -92,7 +91,7 @@ PRE_InitPreempt(struct timeval *slice)
     }
 
 #if HAVE_SIGACTION && defined(SA_SIGINFO)
-    memset((char *)&action, 0, sizeof(action));
+    memset(&action, 0, sizeof(action));
     action.sa_sigaction = AlarmHandler;
     action.sa_flags = SA_SIGINFO;
 
@@ -100,7 +99,7 @@ PRE_InitPreempt(struct timeval *slice)
 	|| (setitimer(ITIMER_REAL, &itv, NULL) == -1))
 	return (LWP_ESYSTEM);
 #else
-    memset((char *)&vec, 0, sizeof(vec));
+    memset(&vec, 0, sizeof(vec));
     vec.sv_handler = AlarmHandler;
     vec.sv_mask = vec.sv_onstack = 0;
 
@@ -128,7 +127,7 @@ PRE_EndPreempt()
     itv.it_value.tv_sec = itv.it_value.tv_usec = 0;
 
 #if HAVE_SIGACTION && defined(SA_SIGINFO)
-    memset((char *)&action, 0, sizeof(action));
+    memset(&action, 0, sizeof(action));
     action.sa_handler = SIG_DFL;
 
     if ((setitimer(ITIMER_REAL, &itv, NULL) == -1)
@@ -136,7 +135,7 @@ PRE_EndPreempt()
 	return (LWP_ESYSTEM);
 
 #else
-    memset((char *)&vec, 0, sizeof(vec));
+    memset(&vec, 0, sizeof(vec));
     vec.sv_handler = SIG_DFL;
     vec.sv_mask = vec.sv_onstack = 0;
 
