@@ -84,7 +84,7 @@ pts_Source(struct cmd_syndesc *as, void *arock)
 	perror(as->parms[0].items->data);
 	return errno;
     }
-    sp = malloc(sizeof *sp);
+    sp = (struct sourcestack *)malloc(sizeof *sp);
     if (!sp) {
 	return errno ? errno : ENOMEM;
     } else {
@@ -123,7 +123,7 @@ popsource(void)
 	fclose(source);
     source = sp->s_file;
     shead = sp->s_next;
-    free(sp);
+    free((char *)sp);
     return 1;
 }
 
@@ -403,7 +403,7 @@ GetNameOrId(struct cmd_syndesc *as, struct idlist *lids, struct namelist *lnames
 	n = 0;			/* count names */
 	for (i = as->parms[0].items; i; i = i->next)
 	    n++;
-	nl->namelist_val = malloc(n * PR_MAXNAMELEN);
+	nl->namelist_val = (prname *) malloc(n * PR_MAXNAMELEN);
 	nl->namelist_len = n;
 	n = 0;
 	for (i = as->parms[0].items; i; i = i->next)
@@ -431,7 +431,7 @@ GetNameOrId(struct cmd_syndesc *as, struct idlist *lids, struct namelist *lnames
 	n = 0;
 	for (i = as->parms[1].items; i; i = i->next)
 	    n++;
-	lids->idlist_val = malloc(n * sizeof(afs_int32));
+	lids->idlist_val = (afs_int32 *) malloc(n * sizeof(afs_int32));
 	lids->idlist_len = n;
 	n = 0;
 	for (i = as->parms[1].items; i; i = i->next) {
@@ -471,30 +471,20 @@ GetNameOrId(struct cmd_syndesc *as, struct idlist *lids,
     struct idlist ids, tids;	/* local copy, if not ret. ids */
     int goodCount = 0;
 
-    /* Initialise our outputs */
-    memset(lids, 0, sizeof(struct idlist));
-    if (lnames)
-	memset(lnames, 0, sizeof(struct namelist));
-
     for (i = as->parms[0].items; i; i = i->next)
 	n++;
-
-    /* Nothing to do, so bail */
-    if (n == 0)
-	return 0;
-
-    lids->idlist_val = malloc(n * sizeof(afs_int32));
+    lids->idlist_val = (afs_int32 *) malloc(n * sizeof(afs_int32));
     lids->idlist_len = n;
-    ids.idlist_val = malloc(n * sizeof(afs_int32));
+    ids.idlist_val = (afs_int32 *) malloc(n * sizeof(afs_int32));
     ids.idlist_len = n;
-    names.namelist_val = malloc(n * PR_MAXNAMELEN);
+    names.namelist_val = (prname *) malloc(n * PR_MAXNAMELEN);
     names.namelist_len = n;
     if (lnames) {
-	lnames->namelist_val = malloc(n * PR_MAXNAMELEN);
+	lnames->namelist_val = (prname *) malloc(n * PR_MAXNAMELEN);
 	lnames->namelist_len = 0;
     }
     for (i = as->parms[0].items; i; i = i->next) {
-	tnames.namelist_val = malloc(PR_MAXNAMELEN);
+	tnames.namelist_val = (prname *) malloc(PR_MAXNAMELEN);
 	strncpy(tnames.namelist_val[0], i->data, PR_MAXNAMELEN);
 	tnames.namelist_len = 1;
 	tids.idlist_len = 0;
@@ -518,7 +508,7 @@ GetNameOrId(struct cmd_syndesc *as, struct idlist *lids,
 	afs_com_err(whoami, code, "so couldn't look up names");
     else {
 	for (n = 0; n < tids.idlist_len; n++) {
-	    if (tids.idlist_val[n] == ANONYMOUSID) {
+	    if ((tids.idlist_val[n] == ANONYMOUSID)) {
 		afs_com_err(whoami, PRNOENT, "so couldn't look up id for %s",
 			names.namelist_val[n]);
 	    } else
@@ -724,7 +714,7 @@ CheckEntry(struct cmd_syndesc *as, void *arock)
 	return PRBADARG;
 
     lids.idlist_len = 2;
-    lids.idlist_val = malloc(sizeof(afs_int32) * 2);
+    lids.idlist_val = (afs_int32 *) malloc(sizeof(afs_int32) * 2);
     lnames.namelist_len = 0;
     lnames.namelist_val = 0;
 
