@@ -353,7 +353,9 @@ CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, af
     newEntry = AllocBlock(at);
     if (!newEntry)
 	return PRDBFAIL;
+#ifdef PR_REMEMBER_TIMES
     tentry.createTime = time(0);
+#endif
 
     if (flag & PRGRP) {
 	tentry.flags = PRGRP;
@@ -405,7 +407,10 @@ CreateEntry(struct ubik_trans *at, char aname[PR_MAXNAMELEN], afs_int32 *aid, af
 	/* To create the user <name>@<cell> the group AUTHUSER_GROUP@<cell>
 	 * must exist.
 	 */
-	asprintf(&cellGroup, "%s%s", AUTHUSER_GROUP, atsign);
+	cellGroup =
+	    (char *)malloc(strlen(AUTHUSER_GROUP) + strlen(atsign) + 1);
+	strcpy(cellGroup, AUTHUSER_GROUP);
+	strcat(cellGroup, atsign);
 	pos = FindByName(at, cellGroup, &centry);
 	free(cellGroup);
 	if (!pos)
@@ -603,7 +608,9 @@ RemoveFromEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 bid)
     code = pr_ReadEntry(at, 0, temp, &tentry);
     if (code != 0)
 	return code;
+#ifdef PR_REMEMBER_TIMES
     tentry.removeTime = time(0);
+#endif
     for (i = 0; i < PRSIZE; i++) {
 	if (tentry.entries[i] == aid) {
 	    tentry.entries[i] = PRBADID;
@@ -759,7 +766,9 @@ RemoveFromSGEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 bid)
     code = pr_ReadEntry(at, 0, temp, &tentry);
     if (code != 0)
 	return code;
+#ifdef PR_REMEMBER_TIMES
     tentry.removeTime = time(NULL);
+#endif
     tentryg = (struct prentryg *)&tentry;
     for (i = 0; i < SGSIZE; i++) {
 	if (tentryg->supergroup[i] == aid) {
@@ -998,7 +1007,9 @@ AddToEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_int3
 
     if (entry->id == aid)
 	return PRINCONSISTENT;
+#ifdef PR_REMEMBER_TIMES
     entry->addTime = time(0);
+#endif
     for (i = 0; i < PRSIZE; i++) {
 	if (entry->entries[i] == aid)
 	    return PRIDEXIST;
@@ -1110,7 +1121,9 @@ AddToSGEntry(struct ubik_trans *tt, struct prentry *entry, afs_int32 loc, afs_in
 
     if (entry->id == aid)
 	return PRINCONSISTENT;
+#ifdef PR_REMEMBER_TIMES
     entry->addTime = time(NULL);
+#endif
     entryg = (struct prentryg *)entry;
     for (i = 0; i < SGSIZE; i++) {
 	if (entryg->supergroup[i] == aid)
@@ -1210,9 +1223,10 @@ AddToPRList(prlist *alist, int *sizeP, afs_int32 id)
     if (alist->prlist_len >= *sizeP) {
 	count = alist->prlist_len + 100;
 	if (alist->prlist_val) {
-	    tmp = realloc(alist->prlist_val, count * sizeof(afs_int32));
+	    tmp =
+		(char *)realloc(alist->prlist_val, count * sizeof(afs_int32));
 	} else {
-	    tmp = malloc(count * sizeof(afs_int32));
+	    tmp = (char *)malloc(count * sizeof(afs_int32));
 	}
 	if (!tmp)
 	    return (PRNOMEM);
@@ -1884,7 +1898,9 @@ ChangeEntry(struct ubik_trans *at, afs_int32 aid, afs_int32 cid, char *name, afs
     if (tentry.owner != cid && !IsAMemberOf(at, cid, SYSADMINID)
 	&& !IsAMemberOf(at, cid, tentry.owner) && !pr_noAuth)
 	return PRPERM;
+#ifdef PR_REMEMBER_TIMES
     tentry.changeTime = time(0);
+#endif
 
     /* we're actually trying to change the id */
     if (newid && (newid != aid)) {
