@@ -53,11 +53,11 @@
 # include "afs/lock.h"
 #else /* KERNEL */
 # include <roken.h>
-# include <afs/opr.h>
 #endif /* KERNEL */
 
 #include "rx.h"
 #include "rx_clock.h"
+#include "rx_queue.h"
 #include "rx_globals.h"
 #include "rx_atomic.h"
 #include "rx_internal.h"
@@ -91,23 +91,23 @@ rxi_GetNextPacket(struct rx_call *call) {
     struct rx_packet *rp;
     int error;
 
-    if (call->app.currentPacket != NULL) {
+    if (call->currentPacket != NULL) {
 #ifdef RX_TRACK_PACKETS
-	call->app.currentPacket->flags |= RX_PKTFLAG_CP;
+	call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
-	rxi_FreePacket(call->app.currentPacket);
-	call->app.currentPacket = NULL;
+	rxi_FreePacket(call->currentPacket);
+	call->currentPacket = NULL;
     }
 
-    if (opr_queue_IsEmpty(&call->rq))
+    if (queue_IsEmpty(&call->rq))
 	return 0;
 
     /* Check that next packet available is next in sequence */
-    rp = opr_queue_First(&call->rq, struct rx_packet, entry);
+    rp = queue_First(&call->rq, rx_packet);
     if (rp->header.seq != call->rnext)
 	return 0;
 
-    opr_queue_Remove(&rp->entry);
+    queue_Remove(rp);
 #ifdef RX_TRACK_PACKETS
     rp->flags &= ~RX_PKTFLAG_RQ;
 #endif
@@ -133,21 +133,21 @@ rxi_GetNextPacket(struct rx_call *call) {
      }
 
     call->rnext++;
-    call->app.currentPacket = rp;
+    call->currentPacket = rp;
 #ifdef RX_TRACK_PACKETS
-    call->app.currentPacket->flags |= RX_PKTFLAG_CP;
+    call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
-    call->app.curvec = 1;	/* 0th vec is always header */
+    call->curvec = 1;	/* 0th vec is always header */
 
     /* begin at the beginning [ more or less ], continue on until the end,
      * then stop. */
-    call->app.curpos = (char *)call->app.currentPacket->wirevec[1].iov_base +
+    call->curpos = (char *)call->currentPacket->wirevec[1].iov_base +
 		   call->conn->securityHeaderSize;
-    call->app.curlen = call->app.currentPacket->wirevec[1].iov_len -
+    call->curlen = call->currentPacket->wirevec[1].iov_len -
 		   call->conn->securityHeaderSize;
 
-    call->app.nLeft = call->app.currentPacket->length;
-    call->app.bytesRcvd += call->app.currentPacket->length;
+    call->nLeft = call->currentPacket->length;
+    call->bytesRcvd += call->currentPacket->length;
 
     call->nHardAcks++;
 
@@ -170,25 +170,25 @@ rxi_ReadProc(struct rx_call *call, char *buf,
     requestCount = nbytes;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
     do {
-	if (call->app.nLeft == 0) {
+	if (call->nLeft == 0) {
 	    /* Get next packet */
 	    MUTEX_ENTER(&call->lock);
 	    for (;;) {
-		if (call->error || (call->app.mode != RX_MODE_RECEIVING)) {
+		if (call->error || (call->mode != RX_MODE_RECEIVING)) {
 		    if (call->error) {
-                        call->app.mode = RX_MODE_ERROR;
+                        call->mode = RX_MODE_ERROR;
 			MUTEX_EXIT(&call->lock);
 			return 0;
 		    }
-		    if (call->app.mode == RX_MODE_SENDING) {
+		    if (call->mode == RX_MODE_SENDING) {
                         MUTEX_EXIT(&call->lock);
 			rxi_FlushWrite(call);
                         MUTEX_ENTER(&call->lock);
@@ -200,7 +200,7 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 		if (code)
 		     return 0;
 
-		if (call->app.currentPacket) {
+		if (call->currentPacket) {
 		    if (!(call->flags & RX_CALL_RECEIVE_DONE)) {
 			if (call->nHardAcks > (u_short) rxi_HardAckRate) {
 			    rxevent_Cancel(&call->delayedAckEvent, call,
@@ -251,45 +251,45 @@ rxi_ReadProc(struct rx_call *call, char *buf,
 	    /* osi_Assert(cp); */
 	    /* MTUXXX  this should be replaced by some error-recovery code before shipping */
 	    /* yes, the following block is allowed to be the ELSE clause (or not) */
-	    /* It's possible for call->app.nLeft to be smaller than any particular
+	    /* It's possible for call->nLeft to be smaller than any particular
 	     * iov_len.  Usually, recvmsg doesn't change the iov_len, since it
 	     * reflects the size of the buffer.  We have to keep track of the
 	     * number of bytes read in the length field of the packet struct.  On
 	     * the final portion of a received packet, it's almost certain that
-	     * call->app.nLeft will be smaller than the final buffer. */
-	    while (nbytes && call->app.currentPacket) {
-		t = MIN((int)call->app.curlen, nbytes);
-		t = MIN(t, (int)call->app.nLeft);
-		memcpy(buf, call->app.curpos, t);
+	     * call->nLeft will be smaller than the final buffer. */
+	    while (nbytes && call->currentPacket) {
+		t = MIN((int)call->curlen, nbytes);
+		t = MIN(t, (int)call->nLeft);
+		memcpy(buf, call->curpos, t);
 		buf += t;
 		nbytes -= t;
-		call->app.curpos += t;
-		call->app.curlen -= t;
-		call->app.nLeft -= t;
+		call->curpos += t;
+		call->curlen -= t;
+		call->nLeft -= t;
 
-		if (!call->app.nLeft) {
+		if (!call->nLeft) {
 		    /* out of packet.  Get another one. */
 #ifdef RX_TRACK_PACKETS
-		    call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		    rxi_FreePacket(call->app.currentPacket);
-		    call->app.currentPacket = NULL;
-		} else if (!call->app.curlen) {
+		    rxi_FreePacket(call->currentPacket);
+		    call->currentPacket = NULL;
+		} else if (!call->curlen) {
 		    /* need to get another struct iov */
-		    if (++call->app.curvec >= call->app.currentPacket->niovecs) {
+		    if (++call->curvec >= call->currentPacket->niovecs) {
 			/* current packet is exhausted, get ready for another */
 			/* don't worry about curvec and stuff, they get set somewhere else */
 #ifdef RX_TRACK_PACKETS
-			call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+			call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-			rxi_FreePacket(call->app.currentPacket);
-			call->app.currentPacket = NULL;
-			call->app.nLeft = 0;
+			rxi_FreePacket(call->currentPacket);
+			call->currentPacket = NULL;
+			call->nLeft = 0;
 		    } else {
-			call->app.curpos =
-			    call->app.currentPacket->wirevec[call->app.curvec].iov_base;
-			call->app.curlen =
-			    call->app.currentPacket->wirevec[call->app.curvec].iov_len;
+			call->curpos =
+			    call->currentPacket->wirevec[call->curvec].iov_base;
+			call->curlen =
+			    call->currentPacket->wirevec[call->curvec].iov_len;
 		    }
 		}
 	    }
@@ -310,28 +310,28 @@ rx_ReadProc(struct rx_call *call, char *buf, int nbytes)
     SPLVAR;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (!queue_IsEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
     /*
      * Most common case, all of the data is in the current iovec.
      * We are relying on nLeft being zero unless the call is in receive mode.
      */
-    if (!call->error && call->app.curlen > nbytes && call->app.nLeft > nbytes) {
-        memcpy(buf, call->app.curpos, nbytes);
+    if (!call->error && call->curlen > nbytes && call->nLeft > nbytes) {
+        memcpy(buf, call->curpos, nbytes);
 
-	call->app.curpos += nbytes;
-	call->app.curlen -= nbytes;
-	call->app.nLeft  -= nbytes;
+	call->curpos += nbytes;
+	call->curlen -= nbytes;
+	call->nLeft  -= nbytes;
 
-        if (!call->app.nLeft && call->app.currentPacket != NULL) {
+        if (!call->nLeft && call->currentPacket != NULL) {
             /* out of packet.  Get another one. */
-            rxi_FreePacket(call->app.currentPacket);
-            call->app.currentPacket = NULL;
+            rxi_FreePacket(call->currentPacket);
+            call->currentPacket = (struct rx_packet *)0;
         }
 	return nbytes;
     }
@@ -350,30 +350,30 @@ rx_ReadProc32(struct rx_call *call, afs_int32 * value)
     SPLVAR;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (!queue_IsEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
     /*
      * Most common case, all of the data is in the current iovec.
      * We are relying on nLeft being zero unless the call is in receive mode.
      */
-    if (!call->error && call->app.curlen >= sizeof(afs_int32)
-	&& call->app.nLeft >= sizeof(afs_int32)) {
+    if (!call->error && call->curlen >= sizeof(afs_int32)
+	&& call->nLeft >= sizeof(afs_int32)) {
 
-        memcpy((char *)value, call->app.curpos, sizeof(afs_int32));
+        memcpy((char *)value, call->curpos, sizeof(afs_int32));
 
-        call->app.curpos += sizeof(afs_int32);
-	call->app.curlen -= sizeof(afs_int32);
-	call->app.nLeft  -= sizeof(afs_int32);
+        call->curpos += sizeof(afs_int32);
+	call->curlen -= sizeof(afs_int32);
+	call->nLeft  -= sizeof(afs_int32);
 
-        if (!call->app.nLeft && call->app.currentPacket != NULL) {
+        if (!call->nLeft && call->currentPacket != NULL) {
             /* out of packet.  Get another one. */
-            rxi_FreePacket(call->app.currentPacket);
-            call->app.currentPacket = NULL;
+            rxi_FreePacket(call->currentPacket);
+            call->currentPacket = (struct rx_packet *)0;
         }
 	return sizeof(afs_int32);
     }
@@ -401,13 +401,13 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
     struct iovec *call_iov;
     struct iovec *cur_iov = NULL;
 
-    if (call->app.currentPacket) {
-	cur_iov = &call->app.currentPacket->wirevec[call->app.curvec];
+    if (call->currentPacket) {
+	cur_iov = &call->currentPacket->wirevec[call->curvec];
     }
     call_iov = &call->iov[call->iovNext];
 
     while (!call->error && call->iovNBytes && call->iovNext < call->iovMax) {
-	if (call->app.nLeft == 0) {
+	if (call->nLeft == 0) {
 	    /* Get next packet */
 	    code = rxi_GetNextPacket(call);
 	    if (code) {
@@ -415,8 +415,8 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	        return 1;
 	    }
 
-	    if (call->app.currentPacket) {
-		cur_iov = &call->app.currentPacket->wirevec[1];
+	    if (call->currentPacket) {
+		cur_iov = &call->currentPacket->wirevec[1];
 		didConsume = 1;
 		continue;
 	    } else {
@@ -424,59 +424,57 @@ rxi_FillReadVec(struct rx_call *call, afs_uint32 serial)
 	    }
 	}
 
-	/* It's possible for call->app.nLeft to be smaller than any particular
+	/* It's possible for call->nLeft to be smaller than any particular
 	 * iov_len.  Usually, recvmsg doesn't change the iov_len, since it
 	 * reflects the size of the buffer.  We have to keep track of the
 	 * number of bytes read in the length field of the packet struct.  On
 	 * the final portion of a received packet, it's almost certain that
-	 * call->app.nLeft will be smaller than the final buffer. */
+	 * call->nLeft will be smaller than the final buffer. */
 	while (call->iovNBytes
 	       && call->iovNext < call->iovMax
-	       && call->app.currentPacket) {
+	       && call->currentPacket) {
 
-	    t = MIN((int)call->app.curlen, call->iovNBytes);
-	    t = MIN(t, (int)call->app.nLeft);
-	    call_iov->iov_base = call->app.curpos;
+	    t = MIN((int)call->curlen, call->iovNBytes);
+	    t = MIN(t, (int)call->nLeft);
+	    call_iov->iov_base = call->curpos;
 	    call_iov->iov_len = t;
 	    call_iov++;
 	    call->iovNext++;
 	    call->iovNBytes -= t;
-	    call->app.curpos += t;
-	    call->app.curlen -= t;
-	    call->app.nLeft -= t;
+	    call->curpos += t;
+	    call->curlen -= t;
+	    call->nLeft -= t;
 
-	    if (!call->app.nLeft) {
+	    if (!call->nLeft) {
 		/* out of packet.  Get another one. */
 #ifdef RX_TRACK_PACKETS
-                call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
-                call->app.currentPacket->flags |= RX_PKTFLAG_IOVQ;
+                call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+                call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-		opr_queue_Append(&call->app.iovq,
-				 &call->app.currentPacket->entry);
+		queue_Append(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                 call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-		call->app.currentPacket = NULL;
-	    } else if (!call->app.curlen) {
+		call->currentPacket = NULL;
+	    } else if (!call->curlen) {
 		/* need to get another struct iov */
-		if (++call->app.curvec >= call->app.currentPacket->niovecs) {
+		if (++call->curvec >= call->currentPacket->niovecs) {
 		    /* current packet is exhausted, get ready for another */
 		    /* don't worry about curvec and stuff, they get set somewhere else */
 #ifdef RX_TRACK_PACKETS
-		    call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
-		    call->app.currentPacket->flags |= RX_PKTFLAG_IOVQ;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-		    opr_queue_Append(&call->app.iovq,
-				     &call->app.currentPacket->entry);
+		    queue_Append(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                     call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-		    call->app.currentPacket = NULL;
-		    call->app.nLeft = 0;
+		    call->currentPacket = NULL;
+		    call->nLeft = 0;
 		} else {
 		    cur_iov++;
-		    call->app.curpos = (char *)cur_iov->iov_base;
-		    call->app.curlen = cur_iov->iov_len;
+		    call->curpos = (char *)cur_iov->iov_base;
+		    call->curlen = cur_iov->iov_len;
 		}
 	    }
 	}
@@ -514,14 +512,14 @@ rxi_ReadvProc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     int bytes;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
-    if (call->app.mode == RX_MODE_SENDING) {
+    if (call->mode == RX_MODE_SENDING) {
 	rxi_FlushWrite(call);
     }
 
@@ -571,7 +569,7 @@ rxi_ReadvProc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 
   error:
     MUTEX_EXIT(&call->lock);
-    call->app.mode = RX_MODE_ERROR;
+    call->mode = RX_MODE_ERROR;
     return 0;
 }
 
@@ -602,25 +600,25 @@ rxi_WriteProc(struct rx_call *call, char *buf,
     int requestCount = nbytes;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
-    if (call->app.mode != RX_MODE_SENDING) {
+    if (call->mode != RX_MODE_SENDING) {
 	if ((conn->type == RX_SERVER_CONNECTION)
-	    && (call->app.mode == RX_MODE_RECEIVING)) {
-	    call->app.mode = RX_MODE_SENDING;
-	    if (call->app.currentPacket) {
+	    && (call->mode == RX_MODE_RECEIVING)) {
+	    call->mode = RX_MODE_SENDING;
+	    if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(call->app.currentPacket);
-		call->app.currentPacket = NULL;
-		call->app.nLeft = 0;
-		call->app.nFree = 0;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
+		call->nLeft = 0;
+		call->nFree = 0;
 	    }
 	} else {
 	    return 0;
@@ -632,32 +630,33 @@ rxi_WriteProc(struct rx_call *call, char *buf,
      * there are 0 bytes on the stream, but we must send a packet
      * anyway. */
     do {
-	if (call->app.nFree == 0) {
+	if (call->nFree == 0) {
 	    MUTEX_ENTER(&call->lock);
             if (call->error)
-                call->app.mode = RX_MODE_ERROR;
-	    if (!call->error && call->app.currentPacket) {
+                call->mode = RX_MODE_ERROR;
+	    if (!call->error && call->currentPacket) {
 		clock_NewTime();	/* Bogus:  need new time package */
 		/* The 0, below, specifies that it is not the last packet:
 		 * there will be others. PrepareSendPacket may
 		 * alter the packet length by up to
 		 * conn->securityMaxTrailerSize */
-		call->app.bytesSent += call->app.currentPacket->length;
-		rxi_PrepareSendPacket(call, call->app.currentPacket, 0);
+		call->bytesSent += call->currentPacket->length;
+		rxi_PrepareSendPacket(call, call->currentPacket, 0);
+#ifdef	AFS_GLOBAL_RXLOCK_KERNEL
                 /* PrepareSendPacket drops the call lock */
                 rxi_WaitforTQBusy(call);
+#endif /* AFS_GLOBAL_RXLOCK_KERNEL */
 #ifdef RX_TRACK_PACKETS
-		call->app.currentPacket->flags |= RX_PKTFLAG_TQ;
+		call->currentPacket->flags |= RX_PKTFLAG_TQ;
 #endif
-		opr_queue_Append(&call->tq,
-				 &call->app.currentPacket->entry);
+		queue_Append(&call->tq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                 call->tqc++;
 #endif /* RXDEBUG_PACKET */
 #ifdef RX_TRACK_PACKETS
-                call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+                call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-                call->app.currentPacket = NULL;
+                call->currentPacket = NULL;
 
 		/* If the call is in recovery, let it exhaust its current
 		 * retransmit queue before forcing it to send new packets
@@ -665,12 +664,12 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 		if (!(call->flags & (RX_CALL_FAST_RECOVER))) {
 		    rxi_Start(call, 0);
 		}
-	    } else if (call->app.currentPacket) {
+	    } else if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(call->app.currentPacket);
-		call->app.currentPacket = NULL;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
 	    }
 	    /* Wait for transmit window to open up */
 	    while (!call->error
@@ -688,35 +687,35 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 		call->startWait = 0;
 #ifdef RX_ENABLE_LOCKS
 		if (call->error) {
-                    call->app.mode = RX_MODE_ERROR;
+                    call->mode = RX_MODE_ERROR;
 		    MUTEX_EXIT(&call->lock);
 		    return 0;
 		}
 #endif /* RX_ENABLE_LOCKS */
 	    }
-	    if ((call->app.currentPacket = rxi_AllocSendPacket(call, nbytes))) {
+	    if ((call->currentPacket = rxi_AllocSendPacket(call, nbytes))) {
 #ifdef RX_TRACK_PACKETS
-		call->app.currentPacket->flags |= RX_PKTFLAG_CP;
+		call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
-		call->app.nFree = call->app.currentPacket->length;
-		call->app.curvec = 1;	/* 0th vec is always header */
+		call->nFree = call->currentPacket->length;
+		call->curvec = 1;	/* 0th vec is always header */
 		/* begin at the beginning [ more or less ], continue
 		 * on until the end, then stop. */
-		call->app.curpos =
-		    (char *) call->app.currentPacket->wirevec[1].iov_base +
+		call->curpos =
+		    (char *) call->currentPacket->wirevec[1].iov_base +
 		    call->conn->securityHeaderSize;
-		call->app.curlen =
-		    call->app.currentPacket->wirevec[1].iov_len -
+		call->curlen =
+		    call->currentPacket->wirevec[1].iov_len -
 		    call->conn->securityHeaderSize;
 	    }
 	    if (call->error) {
-                call->app.mode = RX_MODE_ERROR;
-		if (call->app.currentPacket) {
+                call->mode = RX_MODE_ERROR;
+		if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		    call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		    rxi_FreePacket(call->app.currentPacket);
-		    call->app.currentPacket = NULL;
+		    rxi_FreePacket(call->currentPacket);
+		    call->currentPacket = NULL;
 		}
 		MUTEX_EXIT(&call->lock);
 		return 0;
@@ -724,19 +723,19 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	    MUTEX_EXIT(&call->lock);
 	}
 
-	if (call->app.currentPacket && (int)call->app.nFree < nbytes) {
+	if (call->currentPacket && (int)call->nFree < nbytes) {
 	    /* Try to extend the current buffer */
 	    int len, mud;
-	    len = call->app.currentPacket->length;
+	    len = call->currentPacket->length;
 	    mud = rx_MaxUserDataSize(call);
 	    if (mud > len) {
 		int want;
-		want = MIN(nbytes - (int)call->app.nFree, mud - len);
-		rxi_AllocDataBuf(call->app.currentPacket, want,
+		want = MIN(nbytes - (int)call->nFree, mud - len);
+		rxi_AllocDataBuf(call->currentPacket, want,
 				 RX_PACKET_CLASS_SEND_CBUF);
-		if (call->app.currentPacket->length > (unsigned)mud)
-		    call->app.currentPacket->length = mud;
-		call->app.nFree += (call->app.currentPacket->length - len);
+		if (call->currentPacket->length > (unsigned)mud)
+		    call->currentPacket->length = mud;
+		call->nFree += (call->currentPacket->length - len);
 	    }
 	}
 
@@ -744,31 +743,31 @@ rxi_WriteProc(struct rx_call *call, char *buf,
 	 * and return.  Don't ship a buffer that's full immediately to
 	 * the peer--we don't know if it's the last buffer yet */
 
-	if (!call->app.currentPacket) {
-	    call->app.nFree = 0;
+	if (!call->currentPacket) {
+	    call->nFree = 0;
 	}
 
-	while (nbytes && call->app.nFree) {
+	while (nbytes && call->nFree) {
 
-	    t = MIN((int)call->app.curlen, nbytes);
-	    t = MIN((int)call->app.nFree, t);
-	    memcpy(call->app.curpos, buf, t);
+	    t = MIN((int)call->curlen, nbytes);
+	    t = MIN((int)call->nFree, t);
+	    memcpy(call->curpos, buf, t);
 	    buf += t;
 	    nbytes -= t;
-	    call->app.curpos += t;
-	    call->app.curlen -= (u_short)t;
-	    call->app.nFree -= (u_short)t;
+	    call->curpos += t;
+	    call->curlen -= (u_short)t;
+	    call->nFree -= (u_short)t;
 
-	    if (!call->app.curlen) {
+	    if (!call->curlen) {
 		/* need to get another struct iov */
-		if (++call->app.curvec >= call->app.currentPacket->niovecs) {
+		if (++call->curvec >= call->currentPacket->niovecs) {
 		    /* current packet is full, extend or send it */
-		    call->app.nFree = 0;
+		    call->nFree = 0;
 		} else {
-		    call->app.curpos =
-			call->app.currentPacket->wirevec[call->app.curvec].iov_base;
-		    call->app.curlen =
-			call->app.currentPacket->wirevec[call->app.curvec].iov_len;
+		    call->curpos =
+			call->currentPacket->wirevec[call->curvec].iov_base;
+		    call->curlen =
+			call->currentPacket->wirevec[call->curvec].iov_len;
 		}
 	    }
 	}			/* while bytes to send and room to send them */
@@ -792,26 +791,26 @@ rx_WriteProc(struct rx_call *call, char *buf, int nbytes)
     SPLVAR;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
     /*
      * Most common case: all of the data fits in the current iovec.
      * We are relying on nFree being zero unless the call is in send mode.
      */
-    tcurlen = (int)call->app.curlen;
-    tnFree = (int)call->app.nFree;
+    tcurlen = (int)call->curlen;
+    tnFree = (int)call->nFree;
     if (!call->error && tcurlen >= nbytes && tnFree >= nbytes) {
-	tcurpos = call->app.curpos;
+	tcurpos = call->curpos;
 
 	memcpy(tcurpos, buf, nbytes);
-	call->app.curpos = tcurpos + nbytes;
-	call->app.curlen = (u_short)(tcurlen - nbytes);
-	call->app.nFree = (u_short)(tnFree - nbytes);
+	call->curpos = tcurpos + nbytes;
+	call->curlen = (u_short)(tcurlen - nbytes);
+	call->nFree = (u_short)(tnFree - nbytes);
 	return nbytes;
     }
 
@@ -831,31 +830,31 @@ rx_WriteProc32(struct rx_call *call, afs_int32 * value)
     char *tcurpos;
     SPLVAR;
 
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
     /*
      * Most common case: all of the data fits in the current iovec.
      * We are relying on nFree being zero unless the call is in send mode.
      */
-    tcurlen = call->app.curlen;
-    tnFree = call->app.nFree;
+    tcurlen = call->curlen;
+    tnFree = call->nFree;
     if (!call->error && tcurlen >= sizeof(afs_int32)
 	&& tnFree >= sizeof(afs_int32)) {
-	tcurpos = call->app.curpos;
+	tcurpos = call->curpos;
 
 	if (!((size_t)tcurpos & (sizeof(afs_int32) - 1))) {
 	    *((afs_int32 *) (tcurpos)) = *value;
 	} else {
 	    memcpy(tcurpos, (char *)value, sizeof(afs_int32));
 	}
-	call->app.curpos = tcurpos + sizeof(afs_int32);
-	call->app.curlen = (u_short)(tcurlen - sizeof(afs_int32));
-	call->app.nFree = (u_short)(tnFree - sizeof(afs_int32));
+	call->curpos = tcurpos + sizeof(afs_int32);
+	call->curlen = (u_short)(tcurlen - sizeof(afs_int32));
+	call->nFree = (u_short)(tnFree - sizeof(afs_int32));
 	return sizeof(afs_int32);
     }
 
@@ -891,25 +890,25 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     nextio = 0;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
-    if (call->app.mode != RX_MODE_SENDING) {
+    if (call->mode != RX_MODE_SENDING) {
 	if ((conn->type == RX_SERVER_CONNECTION)
-	    && (call->app.mode == RX_MODE_RECEIVING)) {
-	    call->app.mode = RX_MODE_SENDING;
-	    if (call->app.currentPacket) {
+	    && (call->mode == RX_MODE_RECEIVING)) {
+	    call->mode = RX_MODE_SENDING;
+	    if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-		rxi_FreePacket(call->app.currentPacket);
-		call->app.currentPacket = NULL;
-		call->app.nLeft = 0;
-		call->app.nFree = 0;
+		rxi_FreePacket(call->currentPacket);
+		call->currentPacket = NULL;
+		call->nLeft = 0;
+		call->nFree = 0;
 	    }
 	} else {
 	    return 0;
@@ -917,11 +916,11 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
     }
 
     /* Set up the iovec to point to data in packet buffers. */
-    tnFree = call->app.nFree;
-    tcurvec = call->app.curvec;
-    tcurpos = call->app.curpos;
-    tcurlen = call->app.curlen;
-    cp = call->app.currentPacket;
+    tnFree = call->nFree;
+    tcurvec = call->curvec;
+    tcurpos = call->curpos;
+    tcurlen = call->curlen;
+    cp = call->currentPacket;
     do {
 	int t;
 
@@ -938,7 +937,7 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 #ifdef RX_TRACK_PACKETS
 	    cp->flags |= RX_PKTFLAG_IOVQ;
 #endif
-	    opr_queue_Append(&call->app.iovq, &cp->entry);
+	    queue_Append(&call->iovq, cp);
 #ifdef RXDEBUG_PACKET
             call->iovqc++;
 #endif /* RXDEBUG_PACKET */
@@ -962,8 +961,8 @@ rxi_WritevAlloc(struct rx_call *call, struct iovec *iov, int *nio, int maxio,
 		if (cp->length > (unsigned)mud)
 		    cp->length = mud;
 		tnFree += (cp->length - len);
-		if (cp == call->app.currentPacket) {
-		    call->app.nFree += (cp->length - len);
+		if (cp == call->currentPacket) {
+		    call->nFree += (cp->length - len);
 		}
 	    }
 	}
@@ -1017,11 +1016,11 @@ int
 rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 {
 #ifdef RX_TRACK_PACKETS
-    struct opr_queue *cursor;
+    struct rx_packet *p, *np;
 #endif
     int nextio;
     int requestCount;
-    struct opr_queue tmpq;
+    struct rx_queue tmpq;
 #ifdef RXDEBUG_PACKET
     u_short tmpqc;
 #endif
@@ -1031,31 +1030,32 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 
     MUTEX_ENTER(&call->lock);
     if (call->error) {
-        call->app.mode = RX_MODE_ERROR;
-    } else if (call->app.mode != RX_MODE_SENDING) {
+        call->mode = RX_MODE_ERROR;
+    } else if (call->mode != RX_MODE_SENDING) {
 	call->error = RX_PROTOCOL_ERROR;
     }
+#ifdef AFS_GLOBAL_RXLOCK_KERNEL
     rxi_WaitforTQBusy(call);
+#endif /* AFS_GLOBAL_RXLOCK_KERNEL */
 
     if (call->error) {
-        call->app.mode = RX_MODE_ERROR;
+        call->mode = RX_MODE_ERROR;
 	MUTEX_EXIT(&call->lock);
-	if (call->app.currentPacket) {
+	if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-            call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
-            call->app.currentPacket->flags |= RX_PKTFLAG_IOVQ;
+            call->currentPacket->flags &= ~RX_PKTFLAG_CP;
+            call->currentPacket->flags |= RX_PKTFLAG_IOVQ;
 #endif
-	    opr_queue_Prepend(&call->app.iovq,
-			      &call->app.currentPacket->entry);
+	    queue_Prepend(&call->iovq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
             call->iovqc++;
 #endif /* RXDEBUG_PACKET */
-	    call->app.currentPacket = NULL;
+	    call->currentPacket = NULL;
 	}
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
 	return 0;
     }
 
@@ -1065,30 +1065,32 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
      * the iovec. We put the loop condition at the end to ensure that
      * a zero length write will push a short packet. */
     nextio = 0;
-    opr_queue_Init(&tmpq);
+    queue_Init(&tmpq);
 #ifdef RXDEBUG_PACKET
     tmpqc = 0;
 #endif /* RXDEBUG_PACKET */
     do {
-	if (call->app.nFree == 0 && call->app.currentPacket) {
+	if (call->nFree == 0 && call->currentPacket) {
 	    clock_NewTime();	/* Bogus:  need new time package */
 	    /* The 0, below, specifies that it is not the last packet:
 	     * there will be others. PrepareSendPacket may
 	     * alter the packet length by up to
 	     * conn->securityMaxTrailerSize */
-	    call->app.bytesSent += call->app.currentPacket->length;
-	    rxi_PrepareSendPacket(call, call->app.currentPacket, 0);
+	    call->bytesSent += call->currentPacket->length;
+	    rxi_PrepareSendPacket(call, call->currentPacket, 0);
+#ifdef	AFS_GLOBAL_RXLOCK_KERNEL
             /* PrepareSendPacket drops the call lock */
             rxi_WaitforTQBusy(call);
-	    opr_queue_Append(&tmpq, &call->app.currentPacket->entry);
+#endif /* AFS_GLOBAL_RXLOCK_KERNEL */
+	    queue_Append(&tmpq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
             tmpqc++;
 #endif /* RXDEBUG_PACKET */
-            call->app.currentPacket = NULL;
+            call->currentPacket = NULL;
 
 	    /* The head of the iovq is now the current packet */
 	    if (nbytes) {
-		if (opr_queue_IsEmpty(&call->app.iovq)) {
+		if (queue_IsEmpty(&call->iovq)) {
                     MUTEX_EXIT(&call->lock);
 		    call->error = RX_PROTOCOL_ERROR;
 #ifdef RXDEBUG_PACKET
@@ -1097,44 +1099,41 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
                         rxi_FreePackets(0, &tmpq);
 		    return 0;
 		}
-		call->app.currentPacket
-			= opr_queue_First(&call->app.iovq, struct rx_packet,
-					  entry);
-		opr_queue_Remove(&call->app.currentPacket->entry);
+		call->currentPacket = queue_First(&call->iovq, rx_packet);
+		queue_Remove(call->currentPacket);
 #ifdef RX_TRACK_PACKETS
-                call->app.currentPacket->flags &= ~RX_PKTFLAG_IOVQ;
-		call->app.currentPacket->flags |= RX_PKTFLAG_CP;
+                call->currentPacket->flags &= ~RX_PKTFLAG_IOVQ;
+		call->currentPacket->flags |= RX_PKTFLAG_CP;
 #endif
 #ifdef RXDEBUG_PACKET
                 call->iovqc--;
 #endif /* RXDEBUG_PACKET */
-		call->app.nFree = call->app.currentPacket->length;
-		call->app.curvec = 1;
-		call->app.curpos =
-		    (char *) call->app.currentPacket->wirevec[1].iov_base +
+		call->nFree = call->currentPacket->length;
+		call->curvec = 1;
+		call->curpos =
+		    (char *) call->currentPacket->wirevec[1].iov_base +
 		    call->conn->securityHeaderSize;
-		call->app.curlen =
-		    call->app.currentPacket->wirevec[1].iov_len -
+		call->curlen =
+		    call->currentPacket->wirevec[1].iov_len -
 		    call->conn->securityHeaderSize;
 	    }
 	}
 
 	if (nbytes) {
 	    /* The next iovec should point to the current position */
-	    if (iov[nextio].iov_base != call->app.curpos
-		|| iov[nextio].iov_len > (int)call->app.curlen) {
+	    if (iov[nextio].iov_base != call->curpos
+		|| iov[nextio].iov_len > (int)call->curlen) {
 		call->error = RX_PROTOCOL_ERROR;
                 MUTEX_EXIT(&call->lock);
-		if (call->app.currentPacket) {
+		if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-		    call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+		    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-                    opr_queue_Prepend(&tmpq,
-				      &call->app.currentPacket->entry);
+                    queue_Prepend(&tmpq, call->currentPacket);
 #ifdef RXDEBUG_PACKET
                     tmpqc++;
 #endif /* RXDEBUG_PACKET */
-                    call->app.currentPacket = NULL;
+                    call->currentPacket = NULL;
 		}
 #ifdef RXDEBUG_PACKET
                 tmpqc -=
@@ -1143,18 +1142,18 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
 		return 0;
 	    }
 	    nbytes -= iov[nextio].iov_len;
-	    call->app.curpos += iov[nextio].iov_len;
-	    call->app.curlen -= iov[nextio].iov_len;
-	    call->app.nFree -= iov[nextio].iov_len;
+	    call->curpos += iov[nextio].iov_len;
+	    call->curlen -= iov[nextio].iov_len;
+	    call->nFree -= iov[nextio].iov_len;
 	    nextio++;
-	    if (call->app.curlen == 0) {
-		if (++call->app.curvec > call->app.currentPacket->niovecs) {
-		    call->app.nFree = 0;
+	    if (call->curlen == 0) {
+		if (++call->curvec > call->currentPacket->niovecs) {
+		    call->nFree = 0;
 		} else {
-		    call->app.curpos =
-			call->app.currentPacket->wirevec[call->app.curvec].iov_base;
-		    call->app.curlen =
-			call->app.currentPacket->wirevec[call->app.curvec].iov_len;
+		    call->curpos =
+			call->currentPacket->wirevec[call->curvec].iov_base;
+		    call->curlen =
+			call->currentPacket->wirevec[call->curvec].iov_len;
 		}
 	    }
 	}
@@ -1164,16 +1163,16 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
      * We may end up with more than call->twind packets on the queue. */
 
 #ifdef RX_TRACK_PACKETS
-    for (opr_queue_Scan(&tmpq, cursor))
+    for (queue_Scan(&tmpq, p, np, rx_packet))
     {
-	struct rx_packet *p = opr_queue_Entry(cursor, struct rx_packet, entry);
         p->flags |= RX_PKTFLAG_TQ;
     }
 #endif
-    if (call->error)
-        call->app.mode = RX_MODE_ERROR;
 
-    opr_queue_SpliceAppend(&call->tq, &tmpq);
+    if (call->error)
+        call->mode = RX_MODE_ERROR;
+
+    queue_SpliceAppend(&call->tq, &tmpq);
 
     /* If the call is in recovery, let it exhaust its current retransmit
      * queue before forcing it to send new packets
@@ -1196,14 +1195,14 @@ rxi_WritevProc(struct rx_call *call, struct iovec *iov, int nio, int nbytes)
     }
 
     if (call->error) {
-        call->app.mode = RX_MODE_ERROR;
-        call->app.currentPacket = NULL;
+        call->mode = RX_MODE_ERROR;
+        call->currentPacket = NULL;
         MUTEX_EXIT(&call->lock);
-	if (call->app.currentPacket) {
+	if (call->currentPacket) {
 #ifdef RX_TRACK_PACKETS
-	    call->app.currentPacket->flags &= ~RX_PKTFLAG_CP;
+	    call->currentPacket->flags &= ~RX_PKTFLAG_CP;
 #endif
-	    rxi_FreePacket(call->app.currentPacket);
+	    rxi_FreePacket(call->currentPacket);
 	}
 	return 0;
     }
@@ -1235,16 +1234,16 @@ rxi_FlushWrite(struct rx_call *call)
     struct rx_packet *cp = NULL;
 
     /* Free any packets from the last call to ReadvProc/WritevProc */
-    if (!opr_queue_IsEmpty(&call->app.iovq)) {
+    if (queue_IsNotEmpty(&call->iovq)) {
 #ifdef RXDEBUG_PACKET
         call->iovqc -=
 #endif /* RXDEBUG_PACKET */
-            rxi_FreePackets(0, &call->app.iovq);
+            rxi_FreePackets(0, &call->iovq);
     }
 
-    if (call->app.mode == RX_MODE_SENDING) {
+    if (call->mode == RX_MODE_SENDING) {
 
-	call->app.mode =
+	call->mode =
 	    (call->conn->type ==
 	     RX_CLIENT_CONNECTION ? RX_MODE_RECEIVING : RX_MODE_EOF);
 
@@ -1263,11 +1262,9 @@ rxi_FlushWrite(struct rx_call *call)
 
         MUTEX_ENTER(&call->lock);
         if (call->error)
-            call->app.mode = RX_MODE_ERROR;
+            call->mode = RX_MODE_ERROR;
 
-	call->flags |= RX_CALL_FLUSH;
-
-        cp = call->app.currentPacket;
+        cp = call->currentPacket;
 
 	if (cp) {
 	    /* cp->length is only supposed to be the user's data */
@@ -1276,9 +1273,9 @@ rxi_FlushWrite(struct rx_call *call)
 #ifdef RX_TRACK_PACKETS
 	    cp->flags &= ~RX_PKTFLAG_CP;
 #endif
-	    cp->length -= call->app.nFree;
-	    call->app.currentPacket = NULL;
-	    call->app.nFree = 0;
+	    cp->length -= call->nFree;
+	    call->currentPacket = (struct rx_packet *)0;
+	    call->nFree = 0;
 	} else {
 	    cp = rxi_AllocSendPacket(call, 0);
 	    if (!cp) {
@@ -1287,18 +1284,20 @@ rxi_FlushWrite(struct rx_call *call)
 	    }
 	    cp->length = 0;
 	    cp->niovecs = 2;	/* header + space for rxkad stuff */
-	    call->app.nFree = 0;
+	    call->nFree = 0;
 	}
 
 	/* The 1 specifies that this is the last packet */
-	call->app.bytesSent += cp->length;
+	call->bytesSent += cp->length;
 	rxi_PrepareSendPacket(call, cp, 1);
+#ifdef	AFS_GLOBAL_RXLOCK_KERNEL
         /* PrepareSendPacket drops the call lock */
         rxi_WaitforTQBusy(call);
+#endif /* AFS_GLOBAL_RXLOCK_KERNEL */
 #ifdef RX_TRACK_PACKETS
 	cp->flags |= RX_PKTFLAG_TQ;
 #endif
-	opr_queue_Append(&call->tq, &cp->entry);
+	queue_Append(&call->tq, cp);
 #ifdef RXDEBUG_PACKET
         call->tqc++;
 #endif /* RXDEBUG_PACKET */
