@@ -12,10 +12,6 @@
 #include <afs/stds.h>
 
 #include <roken.h>
-#include <afs/opr.h>
-#ifdef AFS_PTHREAD_ENV
-# include <opr/lock.h>
-#endif
 
 #ifdef IGNORE_SOME_GCC_WARNINGS
 # pragma GCC diagnostic warning "-Wstrict-prototypes"
@@ -97,7 +93,7 @@ static pthread_key_t random_number_key;
 static void
 afs_random_once(void)
 {
-    opr_Verify(pthread_key_create(&random_number_key, NULL) == 0);
+    osi_Assert(pthread_key_create(&random_number_key, NULL) == 0);
     called_afs_random_once = 1;
 }
 
@@ -221,7 +217,7 @@ ubik_ClientInit(struct rx_connection **serverconns,
 	    return UMUTEXDESTROY;
 #endif
     } else {
-	tc = malloc(sizeof(struct ubik_client));
+	tc = (struct ubik_client *)malloc(sizeof(struct ubik_client));
     }
     if (tc == NULL)
 	return UNOMEM;
@@ -326,7 +322,7 @@ ubik_RefreshConn(struct rx_connection *tc)
 pthread_once_t ubik_client_once = PTHREAD_ONCE_INIT;
 pthread_mutex_t ubik_client_mutex;
 #define LOCK_UCLNT_CACHE do { \
-    opr_Verify(pthread_once(&ubik_client_once, ubik_client_init_mutex) == 0); \
+    osi_Assert(pthread_once(&ubik_client_once, ubik_client_init_mutex) == 0); \
     MUTEX_ENTER(&ubik_client_mutex); \
   } while (0)
 #define UNLOCK_UCLNT_CACHE MUTEX_EXIT(&ubik_client_mutex)
@@ -435,8 +431,12 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
     while (*apos < MAXSERVERS) {
 	/* tc is the next conn to try */
 	tc = aclient->conns[*apos];
-	if (!tc)
-	    goto errout;
+	if (!tc) {
+	    if (needlock) {
+		UNLOCK_UBIK_CLIENT(aclient);
+	    }
+	    return UNOSERVERS;
+	}
 
 	if (rx_ConnError(tc)) {
 	    tc = ubik_RefreshConn(tc);
@@ -449,15 +449,22 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
 	    break;		/* this is the desired path */
 	}
     }
-    if (*apos >= MAXSERVERS)
-	goto errout;
+    if (*apos >= MAXSERVERS) {
+	if (needlock) {
+	    UNLOCK_UBIK_CLIENT(aclient);
+	}
+	return UNOSERVERS;
+    }
 
     code =
 	(*aproc) (tc, p1, p2, p3, p4, p5, p6, p7, p8, p9, p10, p11, p12, p13,
 		  p14, p15, p16);
-    if (aclient->initializationState != origLevel)
-	/* somebody did a ubik_ClientInit */
-	goto errout;
+    if (aclient->initializationState != origLevel) {
+	if (needlock) {
+	    UNLOCK_UBIK_CLIENT(aclient);
+	}
+	return code;		/* somebody did a ubik_ClientInit */
+    }
 
     /* what should I do in case of UNOQUORUM ? */
     if (code < 0) {
@@ -468,7 +475,6 @@ CallIter(int (*aproc) (), struct ubik_client *aclient,
     }
 
     (*apos)++;
-errout:
     if (needlock) {
 	UNLOCK_UBIK_CLIENT(aclient);
     }
