@@ -23,18 +23,18 @@
 #include <afs/procmgmt.h>	/* signal(), kill(), wait(), etc. */
 
 #include <roken.h>		/* Must come after procmgmt.h */
-#include <afs/opr.h>
 
-#include <afs/opr.h>
 #include "afsutil.h"
 #include "fileutil.h"
 #include <lwp.h>
 
 #if defined(AFS_PTHREAD_ENV)
+/* can't include rx when we are libutil; it's too early */
+#include <rx/rx.h>
 #include <pthread.h>
 static pthread_mutex_t serverLogMutex;
-#define LOCK_SERVERLOG() opr_Verify(pthread_mutex_lock(&serverLogMutex) == 0);
-#define UNLOCK_SERVERLOG() opr_Verify(pthread_mutex_unlock(&serverLogMutex) == 0);
+#define LOCK_SERVERLOG() MUTEX_ENTER(&serverLogMutex)
+#define UNLOCK_SERVERLOG() MUTEX_EXIT(&serverLogMutex)
 
 #ifdef AFS_NT40_ENV
 #define NULLDEV "NUL"
@@ -159,8 +159,6 @@ LogCommandLine(int argc, char **argv, const char *progname,
     int i, l;
     char *commandLine, *cx;
 
-    opr_Assert(argc > 0);
-
     for (l = i = 0; i < argc; i++)
 	l += strlen(argv[i]) + 1;
     if ((commandLine = malloc(l))) {
@@ -312,7 +310,7 @@ OpenLog(const char *fileName)
 	    goto makefilename;
 	}
 	if (!isfifo)
-	    rk_rename(fileName, FileName);	/* don't check error code */
+	    renamefile(fileName, FileName);	/* don't check error code */
 	tempfd = open(fileName, O_WRONLY | O_TRUNC | O_CREAT | (isfifo?O_NONBLOCK:0), 0666);
     } else {
 	strcpy(oldName, fileName);
@@ -320,7 +318,7 @@ OpenLog(const char *fileName)
 
 	/* don't check error */
 	if (!isfifo)
-	    rk_rename(fileName, oldName);
+	    renamefile(fileName, oldName);
 	tempfd = open(fileName, O_WRONLY | O_TRUNC | O_CREAT | (isfifo?O_NONBLOCK:0), 0666);
     }
 
@@ -338,7 +336,7 @@ OpenLog(const char *fileName)
 #endif
 
 #if defined(AFS_PTHREAD_ENV)
-    opr_Verify(pthread_mutex_init(&serverLogMutex, NULL) == 0);
+    MUTEX_INIT(&serverLogMutex, "serverlog", MUTEX_DEFAULT, 0);
 #endif /* AFS_PTHREAD_ENV */
 
     serverLogFD = tempfd;
