@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -15,11 +15,8 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID
-    ("$Header: /cvs/openafs/src/viced/check_sysid.c,v 1.6.14.1 2007/11/26 21:08:45 shadow Exp $");
+#include <roken.h>
 
-#include <fcntl.h>
-#include <errno.h>
 #include <afs/vldbint.h>
 
 #define SYSIDMAGIC      0x88aabbcc
