@@ -20,12 +20,11 @@
 #include <roken.h>
 
 #include <afs/opr.h>
-#include <opr/lock.h>
 #include <afs/afsint.h>
-#include <rx/rx_queue.h>
 #include "nfs.h"
 #include <afs/errors.h>
 #include "salvsync.h"
+#include "lwp.h"
 #include "lock.h"
 #include <afs/afssyscalls.h>
 #include "ihandle.h"
@@ -94,7 +93,7 @@ SALVSYNC_askSalv(SYNC_command * com, SYNC_response * res)
       break;
     case SYNC_COM_ERROR:
     case SYNC_BAD_COMMAND:
-	Log("SALVSYNC_askSalv: internal SALVSYNC protocol error %d\n", code);
+	Log("SALVSYNC_askSalv: fatal SALVSYNC protocol error; online salvager functionality disabled until next fileserver restart\n");
 	break;
     case SYNC_DENIED:
 	Log("SALVSYNC_askSalv: SALVSYNC request denied for reason=%d\n", res->hdr.reason);
