@@ -271,14 +271,10 @@ extern void VPutVnode(Error * ec, Vnode * vnp);
 extern void VPutVnode_r(Error * ec, Vnode * vnp);
 extern int VVnodeWriteToRead(Error * ec, Vnode * vnp);
 extern int VVnodeWriteToRead_r(Error * ec, Vnode * vnp);
-extern Vnode *VAllocVnode(Error * ec, struct Volume *vp, VnodeType type,
-	VnodeId in_vnode, Unique in_unique);
-extern Vnode *VAllocVnode_r(Error * ec, struct Volume *vp, VnodeType type,
-	VnodeId in_vnode, Unique in_unique);
-
+extern Vnode *VAllocVnode(Error * ec, struct Volume *vp, VnodeType type);
+extern Vnode *VAllocVnode_r(Error * ec, struct Volume *vp, VnodeType type);
 /*extern VFreeVnode();*/
-extern Vnode *VGetFreeVnode_r(struct VnodeClassInfo *vcp, struct Volume *vp,
-                              VnodeId vnodeNumber);
+extern Vnode *VGetFreeVnode_r(struct VnodeClassInfo *vcp);
 extern Vnode *VLookupVnode(struct Volume * vp, VnodeId vnodeId);
 
 extern void AddToVVnList(struct Volume * vp, Vnode * vnp);
