@@ -14,14 +14,9 @@
 
 #include <roken.h>
 
-#include <afs/opr.h>
-#ifdef AFS_PTHREAD_ENV
-# include <opr/lock.h>
-#endif
-
+#include <rx/xdr.h>
 #include <rx/rx.h>
 #include <rx/rxkad.h>
-#include <rx/rx_queue.h>
 #include <afs/afsint.h>
 #include <afs/prs_fs.h>
 #include <afs/nfs.h>
@@ -59,7 +54,6 @@
 
 extern int DoLogging;
 extern struct afsconf_dir *tdir;
-extern int DoPreserveVolumeStats;
 
 extern void LogError(afs_int32 errcode);
 
@@ -339,9 +333,10 @@ ViceCreateRoot(Volume *vp)
     afs_fsize_t length;
     ssize_t nBytes;
 
-    vnode = calloc(1, SIZEOF_LARGEDISKVNODE);
+    vnode = (struct VnodeDiskObject *)malloc(SIZEOF_LARGEDISKVNODE);
     if (!vnode)
 	return ENOMEM;
+    memset(vnode, 0, SIZEOF_LARGEDISKVNODE);
 
     V_pref(vp, nearInode);
     inodeNumber =
@@ -359,7 +354,7 @@ ViceCreateRoot(Volume *vp)
     did.Vnode = (VnodeId) 1;
     did.Unique = 1;
 
-    opr_Verify(!(afs_dir_MakeDir(&dir, (afs_int32 *)&did, (afs_int32 *)&did)));
+    osi_Assert(!(afs_dir_MakeDir(&dir, (afs_int32 *)&did, (afs_int32 *)&did)));
     DFlush();			/* flush all modified dir buffers out */
     DZap(&dir);			/* Remove all buffers for this dir */
     length = afs_dir_Length(&dir);	/* Remember size of this directory */
@@ -399,9 +394,9 @@ ViceCreateRoot(Volume *vp)
     IH_INIT(h, vp->device, V_parentId(vp),
 	    vp->vnodeIndex[vLarge].handle->ih_ino);
     fdP = IH_OPEN(h);
-    opr_Assert(fdP != NULL);
+    osi_Assert(fdP != NULL);
     nBytes = FDH_PWRITE(fdP, vnode, SIZEOF_LARGEDISKVNODE, vnodeIndexOffset(vcp, 1));
-    opr_Assert(nBytes == SIZEOF_LARGEDISKVNODE);
+    osi_Assert(nBytes == SIZEOF_LARGEDISKVNODE);
     FDH_REALLYCLOSE(fdP);
     IH_RELEASE(h);
     VNDISK_GET_LEN(length, vnode);
@@ -416,7 +411,8 @@ SAFSVolPartitionInfo(struct rx_call *acid, char *pname, struct diskPartition
 		     *partition)
 {
     afs_int32 code;
-    struct diskPartition64 *dp = malloc(sizeof(struct diskPartition64));
+    struct diskPartition64 *dp = (struct diskPartition64 *)
+	malloc(sizeof(struct diskPartition64));
 
     code = VolPartitionInfo(acid, pname, dp);
     if (!code) {
@@ -757,6 +753,12 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
 	purgevp = NULL;
     }
     originalvp = tt->volume;
+    if ((V_type(originalvp) == backupVolume)
+	|| (V_type(originalvp) == readonlyVolume)) {
+	Log("1 Volser: Clone: The volume to be cloned must be a read/write; aborted\n");
+	error = EROFS;
+	goto fail;
+    }
     if ((V_destroyMe(originalvp) == DESTROY_ME) || !V_inService(originalvp)) {
 	Log("1 Volser: Clone: Volume %d is offline and cannot be cloned\n",
 	    V_id(originalvp));
@@ -786,17 +788,13 @@ VolClone(struct rx_call *acid, afs_int32 atrans, afs_uint32 purgeId,
     salv_vp = originalvp;
 #endif
 
-    if (purgeId == newId) {
-	newvp = purgevp;
-    } else {
-	newvp =
-	    VCreateVolume(&error, originalvp->partition->name, newId,
-			  V_parentId(originalvp));
-	if (error) {
-	    Log("1 Volser: Clone: Couldn't create new volume; clone aborted\n");
-	    newvp = (Volume *) 0;
-	    goto fail;
-	}
+    newvp =
+	VCreateVolume(&error, originalvp->partition->name, newId,
+		      V_parentId(originalvp));
+    if (error) {
+	Log("1 Volser: Clone: Couldn't create new volume; clone aborted\n");
+	newvp = (Volume *) 0;
+	goto fail;
     }
     if (newType == readonlyVolume)
 	V_cloneId(originalvp) = newId;
@@ -894,7 +892,6 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     afs_int32 newType;
     struct volser_trans *tt, *ttc;
     char caller[MAXKTCNAMELEN];
-    VolumeDiskData saved_header;
 
     /*not a super user */
     if (!afsconf_SuperUser(tdir, acid, caller))
@@ -951,10 +948,6 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
 	goto fail;
     }
 
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&V_disk(clonevp), &saved_header);
-    }
-
     error = 0;
     Log("1 Volser: Clone: Recloning volume %u to volume %u\n", tt->volid,
 	cloneId);
@@ -977,20 +970,15 @@ VolReClone(struct rx_call *acid, afs_int32 atrans, afs_int32 cloneId)
     }
     /* don't do strcpy onto diskstuff.name, it's still OK from 1st clone */
 
-    /* update the creationDate, since this represents the last cloning date
-     * for ROs. But do not update copyDate; let it stay so we can identify
-     * when the clone was first created. */
-    V_creationDate(clonevp) = time(0);
-    if (DoPreserveVolumeStats) {
-	CopyVolumeStats(&saved_header, &V_disk(clonevp));
-    } else {
-	ClearVolumeStats(&V_disk(clonevp));
-    }
+    /* pretend recloned volume is a totally new instance */
+    V_copyDate(clonevp) = time(0);
+    V_creationDate(clonevp) = V_copyDate(clonevp);
+    ClearVolumeStats(&V_disk(clonevp));
     V_destroyMe(clonevp) = 0;
     V_inService(clonevp) = 0;
     if (newType == backupVolume) {
-	V_backupDate(originalvp) = V_creationDate(clonevp);
-	V_backupDate(clonevp) = V_creationDate(clonevp);
+	V_backupDate(originalvp) = V_copyDate(clonevp);
+	V_backupDate(clonevp) = V_copyDate(clonevp);
     }
     V_inUse(clonevp) = 0;
     VUpdateVolume(&error, clonevp);
@@ -1365,7 +1353,8 @@ SAFSVolForwardMultiple(struct rx_call *acid, afs_int32 fromTrans, afs_int32
     if (results) {
 	memset(results, 0, sizeof(manyResults));
 	i = results->manyResults_len = destinations->manyDests_len;
-	results->manyResults_val = codes = malloc(i * sizeof(afs_int32));
+	results->manyResults_val = codes =
+	  (afs_int32 *) malloc(i * sizeof(afs_int32));
     }
     if (!results || !results->manyResults_val)
 	return ENOMEM;
@@ -1386,11 +1375,12 @@ SAFSVolForwardMultiple(struct rx_call *acid, afs_int32 fromTrans, afs_int32
     /* (fromDate == 0) ==> full dump */
     is_incremental = (fromDate ? 1 : 0);
 
-    tcons = malloc(i * sizeof(struct rx_connection *));
+    tcons =
+	(struct rx_connection **)malloc(i * sizeof(struct rx_connection *));
     if (!tcons) {
 	return ENOMEM;
     }
-    tcalls = malloc(i * sizeof(struct rx_call *));
+    tcalls = (struct rx_call **)malloc(i * sizeof(struct rx_call *));
     if (!tcalls) {
 	free(tcons);
 	return ENOMEM;
@@ -1774,7 +1764,7 @@ VolGetName(struct rx_call *acid, afs_int32 atrans, char **aname)
     int len;
 
     /* We need to at least fill it in */
-    *aname = malloc(1);
+    *aname = (char *)malloc(1);
     if (!*aname)
 	return ENOMEM;
     tt = FindTrans(atrans);
@@ -1800,7 +1790,7 @@ VolGetName(struct rx_call *acid, afs_int32 atrans, char **aname)
 	TRELE(tt);
 	return E2BIG;
     }
-    *aname = realloc(*aname, len);
+    *aname = (char *)realloc(*aname, len);
     strcpy(*aname, td->name);
     TClearRxCall(tt);
     if (TRELE(tt))
@@ -1893,7 +1883,7 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 	    partList.partId[j++] = i;
     }
     if (j > 0) {
-	pEntries->partEntries_val = malloc(j * sizeof(int));
+	pEntries->partEntries_val = (afs_int32 *) malloc(j * sizeof(int));
 	if (!pEntries->partEntries_val)
 	    return ENOMEM;
 	memcpy((char *)pEntries->partEntries_val, (char *)&partList,
@@ -1907,30 +1897,28 @@ XVolListPartitions(struct rx_call *acid, struct partEntries *pEntries)
 
 }
 
-/*
- * Scan a directory for possible volume headers.
- * in: DIR *dirp		-- a directory handle from opendir()
- * out: char *volname		-- set to name of directory entry
- *      afs_uint32 *volid	-- set to volume ID parsed from name
- * returns:
- *  true if volname and volid have been set to valid values
- *  false if we got to the end of the directory
- */
-static int
-GetNextVol(DIR *dirp, char *volname, afs_uint32 *volid)
+/*return the name of the next volume header in the directory associated with dirp and dp.
+*the volume id is  returned in volid, and volume header name is returned in volname*/
+int
+GetNextVol(DIR * dirp, char *volname, afs_uint32 * volid)
 {
     struct dirent *dp;
 
-    while ((dp = readdir(dirp)) != NULL) {
-	/* could be optimized on platforms with dp->d_namlen */
-	if (dp->d_name[0] == 'V' && strlen(dp->d_name) == VHDRNAMELEN
-		&& strcmp(&(dp->d_name[11]), VHDREXT) == 0) {
+    dp = readdir(dirp);		/*read next entry in the directory */
+    if (dp) {
+	if ((dp->d_name[0] == 'V') && !strcmp(&(dp->d_name[11]), VHDREXT)) {
 	    *volid = VolumeNumber(dp->d_name);
 	    strcpy(volname, dp->d_name);
-	    return 1;
+	    return 0;		/*return the name of the file representing a volume */
+	} else {
+	    strcpy(volname, "");
+	    return 0;		/*volname doesnot represent a volume */
 	}
+    } else {
+	strcpy(volname, "EOD");
+	return 0;		/*end of directory */
     }
-    return 0;
+
 }
 
 /**
@@ -2338,9 +2326,10 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid,
     int code;
     volint_info_handle_t handle;
 
-    volumeInfo->volEntries_val = calloc(1, sizeof(volintInfo));
+    volumeInfo->volEntries_val = (volintInfo *) malloc(sizeof(volintInfo));
     if (!volumeInfo->volEntries_val)
 	return ENOMEM;
+    memset(volumeInfo->volEntries_val, 0, sizeof(volintInfo)); /* Clear structure */
 
     volumeInfo->volEntries_len = 1;
     if (GetPartName(partid, pname))
@@ -2351,11 +2340,21 @@ VolListOneVolume(struct rx_call *acid, afs_int32 partid,
     if (dirp == NULL)
 	return VOLSERILLEGAL_PARTITION;
 
-    while (GetNextVol(dirp, volname, &volid)) {
+    strcpy(volname, "");
+
+    while (strcmp(volname, "EOD") && !found) {	/*while there are more volumes in the partition */
+
+	if (!strcmp(volname, "")) {	/* its not a volume, fetch next file */
+	    GetNextVol(dirp, volname, &volid);
+	    continue;		/*back to while loop */
+	}
+
 	if (volid == volumeId) {	/*copy other things too */
 	    found = 1;
 	    break;
 	}
+
+	GetNextVol(dirp, volname, &volid);
     }
 
     if (found) {
@@ -2432,9 +2431,11 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
      * Set up our pointers for action, marking our structure to hold exactly
      * one entry.  Also, assume we'll fail in our quest.
      */
-    a_volumeXInfoP->volXEntries_val = calloc(1, sizeof(volintXInfo));
+    a_volumeXInfoP->volXEntries_val =
+	(volintXInfo *) malloc(sizeof(volintXInfo));
     if (!a_volumeXInfoP->volXEntries_val)
 	return ENOMEM;
+    memset(a_volumeXInfoP->volXEntries_val, 0, sizeof(volintXInfo)); /* Clear structure */
 
     a_volumeXInfoP->volXEntries_len = 1;
     code = ENODEV;
@@ -2455,13 +2456,23 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
     if (dirp == NULL)
 	return (VOLSERILLEGAL_PARTITION);
 
+    strcpy(volname, "");
 
     /*
      * Sweep through the partition directory, looking for the desired entry.
      * First, of course, figure out how many stat bytes to copy out of each
      * volume.
      */
-    while (GetNextVol(dirp, volname, &currVolID)) {
+    while (strcmp(volname, "EOD") && !found) {
+	/*
+	 * If this is not a volume, move on to the next entry in the
+	 * partition's directory.
+	 */
+	if (!strcmp(volname, "")) {
+	    GetNextVol(dirp, volname, &currVolID);
+	    continue;
+	}
+
 	if (currVolID == a_volID) {
 	    /*
 	     * We found the volume entry we're interested.  Pull out the
@@ -2471,6 +2482,8 @@ VolXListOneVolume(struct rx_call *a_rxCidP, afs_int32 a_partID,
 	    found = 1;
 	    break;
 	}			/*Found desired volume */
+
+	GetNextVol(dirp, volname, &currVolID);
     }
 
     if (found) {
@@ -2527,9 +2540,11 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
     int code;
     volint_info_handle_t handle;
 
-    volumeInfo->volEntries_val = calloc(allocSize, sizeof(volintInfo));
+    volumeInfo->volEntries_val =
+	(volintInfo *) malloc(allocSize * sizeof(volintInfo));
     if (!volumeInfo->volEntries_val)
 	return ENOMEM;
+    memset(volumeInfo->volEntries_val, 0, sizeof(volintInfo)); /* Clear structure */
 
     pntr = volumeInfo->volEntries_val;
     volumeInfo->volEntries_len = 0;
@@ -2540,8 +2555,15 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
     dirp = opendir(VPartitionPath(partP));
     if (dirp == NULL)
 	return VOLSERILLEGAL_PARTITION;
+    strcpy(volname, "");
+
+    while (strcmp(volname, "EOD")) {	/*while there are more partitions in the partition */
+
+	if (!strcmp(volname, "")) {	/* its not a volume, fetch next file */
+	    GetNextVol(dirp, volname, &volid);
+	    continue;		/*back to while loop */
+	}
 
-    while (GetNextVol(dirp, volname, &volid)) {
 	if (flags) {		/*copy other things too */
 #ifndef AFS_PTHREAD_ENV
 	    IOMGR_Poll();	/*make sure that the client does not time out */
@@ -2557,8 +2579,9 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 			      volname,
 			      &handle,
 			      VOL_INFO_LIST_MULTIPLE);
-	    if (code == -2)	/* DESTROY_ME flag set */
-		continue;
+	    if (code == -2) { /* DESTROY_ME flag set */
+		goto drop2;
+	    }
 	} else {
 	    pntr->volid = volid;
 	    /*just volids are needed */
@@ -2569,8 +2592,9 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 	if ((allocSize - volumeInfo->volEntries_len) < 5) {
 	    /*running out of space, allocate more space */
 	    allocSize = (allocSize * 3) / 2;
-	    pntr = realloc(volumeInfo->volEntries_val,
-			   allocSize * sizeof(volintInfo));
+	    pntr =
+		(volintInfo *) realloc((char *)volumeInfo->volEntries_val,
+				       allocSize * sizeof(volintInfo));
 	    if (pntr == NULL) {
 		closedir(dirp);
 		return VOLSERNO_MEMORY;
@@ -2578,7 +2602,12 @@ VolListVolumes(struct rx_call *acid, afs_int32 partid, afs_int32 flags,
 	    volumeInfo->volEntries_val = pntr;	/* point to new block */
 	    /* set pntr to the right position */
 	    pntr = volumeInfo->volEntries_val + volumeInfo->volEntries_len;
+
 	}
+
+      drop2:
+	GetNextVol(dirp, volname, &volid);
+
     }
 
     closedir(dirp);
@@ -2641,9 +2670,11 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
      * Allocate a large array of extended volume info structures, then
      * set it up for action.
      */
-    a_volumeXInfoP->volXEntries_val = calloc(allocSize, sizeof(volintXInfo));
+    a_volumeXInfoP->volXEntries_val =
+	(volintXInfo *) malloc(allocSize * sizeof(volintXInfo));
     if (!a_volumeXInfoP->volXEntries_val)
 	return ENOMEM;
+    memset(a_volumeXInfoP->volXEntries_val, 0, sizeof(volintXInfo)); /* Clear structure */
 
     xInfoP = a_volumeXInfoP->volXEntries_val;
     a_volumeXInfoP->volXEntries_len = 0;
@@ -2663,7 +2694,23 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
     dirp = opendir(VPartitionPath(partP));
     if (dirp == NULL)
 	return (VOLSERILLEGAL_PARTITION);
-    while (GetNextVol(dirp, volname, &volid)) {
+    strcpy(volname, "");
+
+    /*
+     * Sweep through the partition directory, acting on each entry.  First,
+     * of course, figure out how many stat bytes to copy out of each volume.
+     */
+    while (strcmp(volname, "EOD")) {
+
+	/*
+	 * If this is not a volume, move on to the next entry in the
+	 * partition's directory.
+	 */
+	if (!strcmp(volname, "")) {
+	    GetNextVol(dirp, volname, &volid);
+	    continue;
+	}
+
 	if (a_flags) {
 	    /*
 	     * Full info about the volume desired.  Poll to make sure the
@@ -2682,8 +2729,9 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 			      volname,
 			      &handle,
 			      VOL_INFO_LIST_MULTIPLE);
-	    if (code == -2)	/* DESTROY_ME flag set */
-		continue;
+	    if (code == -2) { /* DESTROY_ME flag set */
+		goto drop2;
+	    }
 	} else {
 	    /*
 	     * Just volume IDs are needed.
@@ -2723,7 +2771,10 @@ VolXListVolumes(struct rx_call *a_rxCidP, afs_int32 a_partID,
 		a_volumeXInfoP->volXEntries_val +
 		a_volumeXInfoP->volXEntries_len;
 	}
-    }
+
+      drop2:
+	GetNextVol(dirp, volname, &volid);
+    }				/*Sweep through the partition directory */
 
     /*
      * We've examined all entries in the partition directory.  Close it,
@@ -2754,7 +2805,7 @@ VolMonitor(struct rx_call *acid, transDebugEntries *transInfo)
     struct volser_trans *tt, *nt, *allTrans;
 
     transInfo->transDebugEntries_val =
-	malloc(allocSize * sizeof(transDebugInfo));
+	(transDebugInfo *) malloc(allocSize * sizeof(transDebugInfo));
     if (!transInfo->transDebugEntries_val)
 	return ENOMEM;
     pntr = transInfo->transDebugEntries_val;
@@ -2792,8 +2843,11 @@ VolMonitor(struct rx_call *acid, transDebugEntries *transInfo)
 	transInfo->transDebugEntries_len += 1;
 	if ((allocSize - transInfo->transDebugEntries_len) < 5) {	/*alloc some more space */
 	    allocSize = (allocSize * 3) / 2;
-	    pntr = realloc(transInfo->transDebugEntries_val,
-			   allocSize * sizeof(transDebugInfo));
+	    pntr =
+		(transDebugInfo *) realloc((char *)transInfo->
+					   transDebugEntries_val,
+					   allocSize *
+					   sizeof(transDebugInfo));
 	    transInfo->transDebugEntries_val = pntr;
 	    pntr =
 		transInfo->transDebugEntries_val +
@@ -2947,9 +3001,15 @@ SAFSVolConvertROtoRWvolume(struct rx_call *acid, afs_int32 partId,
     dirp = opendir(VPartitionPath(partP));
     if (dirp == NULL)
 	return VOLSERILLEGAL_PARTITION;
+    strcpy(volname, "");
     ttc = (struct volser_trans *)0;
 
-    while (GetNextVol(dirp, volname, &volid)) {
+    while (strcmp(volname, "EOD")) {
+	if (!strcmp(volname, "")) {     /* its not a volume, fetch next file */
+            GetNextVol(dirp, volname, &volid);
+            continue;           /*back to while loop */
+        }
+
 	if (volid == volumeId) {        /*copy other things too */
 #ifndef AFS_PTHREAD_ENV
             IOMGR_Poll();       /*make sure that the client doesnot time out */
@@ -2965,6 +3025,7 @@ SAFSVolConvertROtoRWvolume(struct rx_call *acid, afs_int32 partId,
 #endif
 	    break;
 	}
+	GetNextVol(dirp, volname, &volid);
     }
 
     if (ttc) {
